/***************************************************************************
 *   Copyright (C) 2011-2014 by Flukz Team                                 *
 *   Contact : see our web site http://flukz.org/                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef BROWSER_WIDGET_H
#define BROWSER_WIDGET_H

#include <QWidget>
#include <QUrl>
#include <QMap>

#include "../networkengine/message.h"

class QLineEdit;
class QTextBrowser;
class QTextEdit;
class ClientObject;
class SoundEngine;

class BrowserWidget : public QWidget {
  Q_OBJECT
	
  public:
	explicit BrowserWidget(QWidget* parent = 0);
	static QUrl deduceAbsoluteUrl(const QUrl& baseUrl, const QUrl& relativeUrl);
	QStringList filesDependencyList(const QUrl& url);
	void setGameCreditsList(QStringList creditsList);
	
	//read the file containing the map (language code, language name)
	void readLanguageFile(QString const& shortFileName);
	
	//sound engine used to play wav files from the browser
	//this is only a copy from the pointer in flukzwidget (bad practice)
	SoundEngine *soundEngine;
	
  public slots:
	void openLink(const QUrl& link);
	void downloadEnd(const QUrl& url);
	void connectionError(Message::ErrorCode errorCode, const QUrl& url, bool isEditable);
	void startEditing();
	void stopEditing();
	void startPreview();
	void saveTemporaryFile();
	
  signals:
	void askPage(const QUrl& url);
	void textChanged();
	void statusMessage(const QString& message);
	
  private:
	QTextBrowser *internalBrowser;
	QTextEdit *internalEdit;
	
	QUrl currentUrl;
	bool isExistingPage;
	
	QStringList specialStrings, specialStringsTranslations;
	
	QString getTargetFileName(const QString& shortFileName);
	QString getOldFileName(QString basePath, int oldIndex);
	QString getDiffFileName(QString basePath, int oldIndex);
	QString getPageUserName(const QString& shortFileName);
	
	QString homePage();
	void formatToStandardHtml(QString & wikiString);		//transform a string written with wiki syntax in a standard html string
	void createLeftMenu(QString & wikiString);
	void addLeftMenu(QString & wikiString, QString languageLinks, bool isErrorPage=false);
	
	QString linkString(const QString& address, const QString& displayedString) {
		return QString("<a href=\"%1\">%2</a>").arg(address).arg(displayedString);
	}
	
	//--- history of a file and diff ---
	QString createDiffHtml(const QString& oldFileName, const QString& newFileName);
	QString createHistoryHtml(const QString& localFileName, const QString& shortFileName);  //used to display a log file in a readable format
	int oldFileRevision(const QString& shortFileName);
	
	//--- game credits ---
	QString createGameCredits();
	QStringList gameCreditsList;
	
	//internal object to store the language name associated to a language code
	QMap<QString, QString> languageMap;
	QString nativeNameOf(const QString &languageCode);
	
  private slots:
	void setContent(const QUrl& url, bool htmlActive);
	void setContentPreview(const QUrl& url);
	void highlightLink(const QUrl& link);
};

#endif
