/***************************************************************************
 *   Copyright (C) 2011-2014 by Flukz Team                                 *
 *   Contact : see our web site http://flukz.org/                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "commitwidget.h"
#include "../networkengine/filerepository.h"

#include <QHBoxLayout>
#include <QGridLayout>
#include <QPushButton>
#include <QLabel>
#include <QLineEdit>
#include <QFileInfo>
#include <QFileDialog>
#include <QDir>
#include <QTextStream>

QString CommitWidget::lastUsedDirectory;

CommitWidget::CommitWidget(QWidget* parent) : QDialog( parent) {
	//--------------horizontal layout with "Ok" button------------
	QWidget *buttons = new QWidget( this );
	QHBoxLayout *buttonsLayout = new QHBoxLayout(buttons);
	buttonsLayout->setMargin(0);
	
	//Save button and Cancel button (with focus)
	buttonsLayout->addStretch(1);
	
	confirmButton = new QPushButton( QString(""), buttons );
	confirmButton->setAutoDefault(false);
	connect(confirmButton, SIGNAL(clicked()), this, SLOT(commitValidation()));
	buttonsLayout->addWidget(confirmButton);
	
	QPushButton *pushButton = new QPushButton( tr("Cancel"), buttons );
	pushButton->setAutoDefault(false);
	connect(pushButton, SIGNAL(clicked()), this, SLOT(reject()));
	buttonsLayout->addWidget(pushButton);
	
	buttonsLayout->addStretch(1);
	
	//------------------vertical layout --------------------------
	QGridLayout *gridLayout = new QGridLayout( this );
	gridLayout->setSpacing(5);
	gridLayout->setMargin(5);
	int row=0;
	
	//label and button to choose the file to upload
	titleLabel1 = new QLabel;
	titleLabel1->setText(tr("File to upload:"));
	
	localFileName = new QLineEdit;
	
	chooseFileButton = new QPushButton(tr("Choose the file to upload..."));
	chooseFileButton->setAutoDefault(false);
	chooseFileButton->setFixedSize(chooseFileButton->sizeHint());
	connect( chooseFileButton, SIGNAL(clicked()), this, SLOT(chooseFile()) );
	
	gridLayout->addWidget(titleLabel1, row, 0);
	gridLayout->addWidget(localFileName, row, 1);
	gridLayout->addWidget(chooseFileButton, row, 2);
	row++;
	
	//label and button to choose the name on the server
	titleLabel2 = new QLabel;			//text is set in setDialogType
	serverFileName = new QLineEdit;
	
	gridLayout->addWidget(titleLabel2, row, 0);
	gridLayout->addWidget(serverFileName, row, 1);
	row++;
	
	messageLabel = new QLabel;
	
	//commit comment
	QLabel *commentLabel = new QLabel;
	commentLabel->setText(tr("Comment:  "));
	comment = new QLineEdit;
	
	gridLayout->addWidget(messageLabel, row, 0, 1, 3 );
	row++;
	gridLayout->addWidget(commentLabel, row, 0);
	gridLayout->addWidget(comment, row, 1);
	row++;
	
	//---------- meta informations for uploaded files ------
	messageLabelB = new QLabel;
	gridLayout->addWidget(messageLabelB, row, 0, 1, 3);
	row++;
	
	//file author
	authorLabel = new QLabel;
	authorLabel->setText(tr("Author: "));
	author = new QLineEdit;
	gridLayout->addWidget(authorLabel, row, 0);
	gridLayout->addWidget(author, row, 1);
	row++;
	
	//file licence
	licenseLabel = new QLabel;
	licenseLabel->setText(tr("License: "));
	license = new QLineEdit;
	gridLayout->addWidget(licenseLabel, row, 0);
	gridLayout->addWidget(license, row, 1);
	row++;
	
	//source link
	sourceLabel = new QLabel;
	sourceLabel->setText(tr("Source link: "));
	sourceLink = new QLineEdit;
	gridLayout->addWidget(sourceLabel, row, 0);
	gridLayout->addWidget(sourceLink, row, 1);
	row++;

	//label for displaying operation result
	explainLabel = new QLabel;
	gridLayout->addWidget(explainLabel, row, 0, 1, 3);
	row++;
	
	gridLayout->addWidget(buttons, row, 0, 1, 3 );
	
	connect(localFileName, SIGNAL(returnPressed()), serverFileName, SLOT(setFocus()));
	connect(serverFileName, SIGNAL(returnPressed()), comment, SLOT(setFocus()));
	connect(author, SIGNAL(returnPressed()), license, SLOT(setFocus()));
	connect(license, SIGNAL(returnPressed()), sourceLink, SLOT(setFocus()));
	connect(sourceLink, SIGNAL(returnPressed()), this, SLOT(commitValidation()));
	
	//internal state, we are not waiting any network answer
	//(confirm button and return key are active)
	waitingNetworkAnswer = false;
	sendingMetaData = false;
	
	//no context hint button (by setting to 0 the bit of the hint button in the window flag)
	setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
}

void CommitWidget::setDialogType(DialogPossibleKind dialogKindInit, bool userIsLogged) {
	dialogKind = dialogKindInit;
	
	QString messageText;
	if(!userIsLogged) {
		messageText = tr("WARNING : you are not logged as a user.\nYour contribution will be identified with your IP address.\n\n");
	}
	
	if(dialogKind == UploadDialog) {
		setWindowTitle(tr("Upload a file"));
		titleLabel2->setText(tr("Name after upload:"));
		messageText += tr("Please enter a comment describing the file.");
		messageLabel->setText(messageText);
		messageLabelB->setText(tr("\nPlease enter the information about the original author of the work and the license.\nWarning : GPL, CC-BY, CC-BY-SA licenses are accepted, but not CC-BY-NC, CC-BY-NC-SA, CC-BY-NC-ND, CC-BY-ND.\nWe accept only works with no restrictions on the commercial usage or the creation of derivative."));
		confirmButton->setText(tr("Upload"));
		localFileName->setFocus();
	} else
	if(dialogKind == CommitDialog) {
		setWindowTitle(tr("Save confirmation"));
		messageText += tr("Your changes will be saved on the server.\nPlease enter a comment describing the changes.");
		messageLabel->setText(messageText);
		confirmButton->setText(tr("Save"));
		comment->setFocus();
	} else
	if(dialogKind == RenameDialog) {
		setWindowTitle(tr("Rename confirmation"));
		titleLabel2->setText(tr("New name:"));
		messageText += tr("The current file will be renamed.\nPlease explain the reason.");
		messageLabel->setText(messageText);
		confirmButton->setText(tr("Rename"));
		serverFileName->setFocus();
	} else
	if(dialogKind == DeleteDialog) {
		setWindowTitle(tr("Delete confirmation"));
		messageText += tr("The current file will be deleted from the server.\nPlease explain the reason.");
		messageLabel->setText(messageText);
		confirmButton->setText(tr("Delete"));
		comment->setFocus();
	} else
	if(dialogKind == RevertDialog) {
		setWindowTitle(tr("Revert confirmation"));
		messageText += tr("The current file will be reverted to revision %1.\nPlease explain the reason.").arg(commitMessage.fileRevision);
		messageLabel->setText(messageText);
		confirmButton->setText(tr("Revert"));
		comment->setFocus();
	}
	
	if(dialogKind == UploadDialog) {
		connect(comment, SIGNAL(returnPressed()), author, SLOT(setFocus()));
	} else {
		connect(comment, SIGNAL(returnPressed()), this, SLOT(commitValidation()));
	}
	
	titleLabel1->setVisible(dialogKind == UploadDialog);
	localFileName->setVisible(dialogKind == UploadDialog);
	chooseFileButton->setVisible(dialogKind == UploadDialog);
	titleLabel2->setVisible(dialogKind == UploadDialog || dialogKind == RenameDialog);
	serverFileName->setVisible(dialogKind == UploadDialog || dialogKind == RenameDialog);
	messageLabelB->setVisible(dialogKind == UploadDialog);
	licenseLabel->setVisible(dialogKind == UploadDialog);
	license->setVisible(dialogKind == UploadDialog);
	authorLabel->setVisible(dialogKind == UploadDialog);
	author->setVisible(dialogKind == UploadDialog);
	sourceLabel->setVisible(dialogKind == UploadDialog);
	sourceLink->setVisible(dialogKind == UploadDialog);
	//delete any old comment
	comment->setText("");
}

void CommitWidget::setTargetFile(const QString& targetFileName) {
	if(dialogKind == CommitDialog) {
		serverFileName->setText(targetFileName);
	}
}

void CommitWidget::chooseFile() {
	//choose a file
	if(lastUsedDirectory.isEmpty()) {
		lastUsedDirectory = QDir::homePath();
	}
	QString uploadFileName = QFileDialog::getOpenFileName(this, tr("Choose the file to upload"), lastUsedDirectory);
	localFileName->setText(uploadFileName);
	QFileInfo fileInfo(uploadFileName);
	serverFileName->setText(fileInfo.baseName()+"."+fileInfo.completeSuffix());
	serverFileName->setFocus();
	
	//if the used directory is valid, remember it
	QDir usedDirectory(fileInfo.absolutePath());
	if(usedDirectory.exists()) {
		lastUsedDirectory = usedDirectory.absolutePath();
	}
}

//return a string containing the meta data given by the user
//format is for .meta file
QString CommitWidget::metaData() {
	QString result;
	result += comment->text() + "\n\n";
	result += QString("#AUTHOR: ") + author->text() + "\n";
	result += QString("#LICENSE: ") + license->text() + "\n";
	if(!sourceLink->text().isEmpty()) {
		result += QString("#SOURCELINK: [[%1]]\n").arg(sourceLink->text());
	}
	return result;
}

void CommitWidget::commitValidation() {
	//if we are waiting a network answer, we ignore new commit validations
	//this can happen if the return key is pressed multiple times
	if(waitingNetworkAnswer) return;
	
	if(dialogKind==UploadDialog) {
		if(localFileName->text().isEmpty()) {
			localFileName->setFocus();
			return;
		}
		
		//check that the local file exists
		QFileInfo fileInfo(localFileName->text());
		if(!fileInfo.exists()) {
			localFileName->setFocus();
			return;
		}
	}
	
	if(dialogKind==UploadDialog || dialogKind==RenameDialog) {
		if(serverFileName->text().isEmpty()) {
			serverFileName->setFocus();
			return;
		}
	}
	
	//note : we accept empty comments only for ordinary changes and reverts (comment needed for file upload, rename, delete)
	if(dialogKind!=CommitDialog && dialogKind != RevertDialog) {
		if(comment->text().isEmpty()) {
			comment->setFocus();
			return;
		}
	}
	
	if(dialogKind==UploadDialog) {
		//author and license are needed (source link is optional)
		if(author->text().isEmpty()) {
			author->setFocus();
			return;
		}
		if(license->text().isEmpty()) {
			license->setFocus();
			return;
		}
	}
	
	commitMessage.fileName = serverFileName->text();
	commitMessage.msg = comment->text();
	
	if(dialogKind==UploadDialog) {
		commitMessage.absoluteFileName = localFileName->text();
		commitMessage.commandCode = Message::File;
	} else
	if(dialogKind==RenameDialog) {
		//replace the spaces of the new name by an underscore
		commitMessage.fileNameB = FileRepository::nameWithoutSpaces(serverFileName->text());
		commitMessage.commandCode = Message::RenameFile;
	} else
	if(dialogKind==DeleteDialog) {
		commitMessage.commandCode = Message::DeleteFile;
	} else
	if(dialogKind==RevertDialog) {
		commitMessage.commandCode = Message::RevertFile;
	} else
	if(dialogKind==CommitDialog) {
		commitMessage.commandCode = Message::File;
		commitMessage.setPreviousHash(serverFileName->text());
		
		//in case of a normal commit, the content saved by browserwidget or gamewidget is in a temporary file
		commitMessage.absoluteFileName = FileRepository::temporaryFileName();
	}
	
	//before emitting the message to the network, we lock the confirm button and return key
	confirmButton->setEnabled(false);
	waitingNetworkAnswer = true;		//new validations by the return key will be ignored
	explainLabel->setText("");
	
	//the message will be sent to clientObject, and then to the network
	emit askCommit(commitMessage);
}

void CommitWidget::sendMetaData() {
	//create the message to be sent on the network
	commitMessage.fileName = serverFileName->text() + FileRepository::metaResource;
	commitMessage.msg = comment->text();
	commitMessage.absoluteFileName = FileRepository::temporaryFileName();
	commitMessage.commandCode = Message::File;
	
	//write the meta data in the local temporary file
	QFile file( commitMessage.absoluteFileName );
	if ( !file.open( QIODevice::WriteOnly ) ) return;
	QTextStream text( &file );
	text.setCodec("UTF-8");
	text << metaData();
	file.close();
	
	//send the message
	emit askCommit(commitMessage);
}

//receive command codes from the network
//TODO : display successfull messages
void CommitWidget::networkMessage(Message message) {
	if(message.statusCode==Message::ActionSuccess && message.commandCode==Message::RenameFile) {
		//file rename was successfull
		accept();
	} else
	if(message.commandCode==Message::FileReceived) {
		if(dialogKind==UploadDialog && !sendingMetaData) {
			//file upload was successfull
			//we now send the corresponding meta data
			sendingMetaData = true;
			sendMetaData();
		} else {
			//both the main file and the meta data have been sent successfully
			accept();
		}
	} else
	if(message.statusCode==Message::ActionSuccess && message.commandCode==Message::DeleteFile) {
		//file deletion was successfull
		accept();
	} else
	if(message.statusCode==Message::ActionSuccess && message.commandCode==Message::RevertFile) {
		//file revert was successfull
		accept();
	} else
	if(message.statusCode==Message::ActionFailed && message.commandCode==Message::RenameFile) {
		explainLabel->setText(tr("File rename failed."));
		waitingNetworkAnswer = false;
		confirmButton->setEnabled(true);
	} else
	if(message.statusCode==Message::ActionFailed && message.commandCode==Message::DeleteFile) {
		explainLabel->setText(tr("File delete failed."));
		waitingNetworkAnswer = false;
		confirmButton->setEnabled(true);
	} else
	if(message.statusCode==Message::ActionFailed && message.commandCode==Message::RevertFile) {
		explainLabel->setText(tr("File revert failed."));
		waitingNetworkAnswer = false;
		confirmButton->setEnabled(true);
	} else
	if(message.commandCode==Message::WritingForbidden || message.commandCode==Message::WritingFailure) {
		if(dialogKind==CommitDialog) {
			explainLabel->setText(tr("The changes could not be saved."));
		} else {
			explainLabel->setText(tr("File import failed."));
		}
		waitingNetworkAnswer = false;
		confirmButton->setEnabled(true);
	}
}
