/***************************************************************************
 *   Copyright (C) 2011-2014 by Flukz Team                                 *
 *   Contact : see our web site http://flukz.org/                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "flukzwidget.h"
#include "browserwidget.h"
#include "commitwidget.h"
#include "loginwidget.h"
#include "../tools/simpledialogwidget.h"
#include "../gameengine/gamewidget.h"
#include "../soundengine/soundengine.h"
#include "../networkengine/clientobject.h"
#include "../networkengine/filerepository.h"

#include <QHBoxLayout>
#include <QPushButton>
#include <QLineEdit>
#include <QMenu>
#include <QAction>
#include <QMessageBox>
#include <QFileDialog>
#include <QDir>
#include <QTranslator>
#include <QCoreApplication>
#include <QSettings>
#include <QCloseEvent>
#include <QStatusBar>
#include <QTimer>
QString FlukzWidget::flukzVersionNumber("0.4");

//Constructor of Flukz main interface
FlukzWidget::FlukzWidget( QWidget *parent) : QMainWindow( parent ) {
	setWindowIcon(QIcon(QString(":/images/flukz-24x32.png") ));		//icon on the upper left corner and in the taskbar
	setWindowTitle(QString("Flukz ") + flukzVersionNumber);			//window title
	
	//init the local cache for downloaded files
	FileRepository::initRepo(/*isClientSide*/ true);
	
	//--------------------------- horizontal bar --------------------
	//backward button, forward button, address bar
	QHBoxLayout *hboxLayout = new QHBoxLayout;
	backwardArrow = new QPushButton;
	backwardArrow->setIcon(QIcon(":/images/backwardarrow.png"));
	backwardArrow->setEnabled(false);
	connect(backwardArrow, SIGNAL(clicked()), this, SLOT(goToPreviousPage()));
	hboxLayout->addWidget(backwardArrow);
	
	forwardArrow = new QPushButton;
	forwardArrow->setIcon(QIcon(":/images/forwardarrow.png"));
	forwardArrow->setEnabled(false);
	connect(forwardArrow, SIGNAL(clicked()), this, SLOT(goToNextPage()));
	hboxLayout->addWidget(forwardArrow);
	
	//address bar
	addressBar = new QLineEdit(this);
	connect(addressBar, SIGNAL(returnPressed()), this, SLOT(adressBarChanged()));
	connect(addressBar, SIGNAL(textEdited(const QString & )),this, SLOT(AddressBarTextEdited()));
	hboxLayout->addWidget(addressBar);
	
	//refresh button
	refreshButton = new QPushButton;
	refreshButton->setIcon(QIcon(":/images/refresh.png"));
	connect(refreshButton, SIGNAL(clicked()), this, SLOT(refreshButtonClicked()));
	hboxLayout->addWidget(refreshButton);
	
	//edit button
	editButton = new QPushButton;
	editButton->setIcon(QIcon(":/images/edit.png"));
	connect(editButton, SIGNAL(clicked()), this, SLOT(editButtonClicked()));
	hboxLayout->addWidget(editButton);
	isEditing=false;
	
	//save button
	saveButton = new QPushButton;
	saveButton->setIcon(QIcon(":/images/save.png"));
	saveButton->setEnabled(false);
	connect(saveButton, SIGNAL(clicked()), this, SLOT(saveCurrentFile()));
	hboxLayout->addWidget(saveButton);
	isSaved=true;
	
	//create a menu for the tool button
	QMenu *menu = new QMenu ( this );
	
	viewMetaAction = new QAction( this );
	connect( viewMetaAction, SIGNAL( triggered() ), this, SLOT(viewMeta()) );
	menu->addAction(viewMetaAction);
	
	saveCopyAction = new QAction( this );
	connect( saveCopyAction, SIGNAL( triggered() ), this, SLOT(saveCopy()) );
	menu->addAction(saveCopyAction);
	
	menu->addSeparator();
	
	renamePageAction = new QAction( this );
	connect( renamePageAction, SIGNAL( triggered() ), this, SLOT(renameCurrentPage()) );
	menu->addAction(renamePageAction);
	
	deletePageAction = new QAction( this );
	connect( deletePageAction, SIGNAL( triggered() ), this, SLOT(deleteCurrentPage()) );
	menu->addAction(deletePageAction);
	
	menu->addSeparator();
	
	uploadFileAction = new QAction( this );
	connect( uploadFileAction, SIGNAL( triggered() ), this, SLOT(uploadFile()) );
	menu->addAction(uploadFileAction);
	
	//tool button
	toolButton = new QPushButton;
	toolButton->setMenu(menu);
	toolButton->setIcon(QIcon(":/images/tool.png"));
	hboxLayout->addWidget(toolButton);
	
	//create a menu for the user button
	menu = new QMenu ( this );
	
	userLoginAction = new QAction( this );
	connect( userLoginAction, SIGNAL( triggered() ), this, SLOT(userLogin()) );
	menu->addAction(userLoginAction);
	
	userPageAction = new QAction( this );
	userPageAction->setEnabled(false);
	connect( userPageAction, SIGNAL( triggered() ), this, SLOT(userPersonalPage()) );
	menu->addAction(userPageAction);
	
	//user button
	userButton = new QPushButton;
	userButton->setMenu(menu);
	userButton->setIcon(QIcon(":/images/anonymous-user.png"));
	hboxLayout->addWidget(userButton);
	userLoggedIn=false;
	
	//browser
	browserWidget = new BrowserWidget(this);;
	connect(browserWidget, SIGNAL(askPage(const QUrl&)), this, SLOT(pageAsked(const QUrl&)) );
	connect(browserWidget, SIGNAL(textChanged()), this, SLOT(fileChanged()) );
	connect(browserWidget, SIGNAL(statusMessage(const QString&)), this, SLOT(setPermanentStatusMessage(const QString&)) );
	
	//gamewidget
	gameWidget = new GameWidget(this);
	gameWidget->setVisible(false);
	connect(gameWidget, SIGNAL(sceneChanged()), this, SLOT(fileChanged()) );
	connect(gameWidget, SIGNAL(askPage(const QString&)), this, SLOT(gamePageAsked(const QString&)) );
	connect(gameWidget, SIGNAL(gameTestFinished()), this, SLOT(closeGameTestMode()) );
	connect(gameWidget, SIGNAL(askScoreCommit(const QString&, int)), this, SLOT(commitGameScore(const QString&, int)) );
	
	//internal client to contact flukz servers
	clientObject = new ClientObject(this);
	connect(clientObject, SIGNAL(downloadEnd(const QStringList&)), this, SLOT(downloadEnd(const QStringList&)) );
	connect(clientObject, SIGNAL(connectionError(Message::ErrorCode)), this, SLOT(connectionError(Message::ErrorCode)) );
	connect(clientObject, SIGNAL(permanentStatusMessage(const QString&)), this, SLOT(setPermanentStatusMessage(const QString&)) );
	connect(clientObject, SIGNAL(temporaryStatusMessage(const QString&)), this, SLOT(setTemporaryStatusMessage(const QString&)) );
	
	//central widget and layout
	QWidget * widget_central = new QWidget( this );
	setCentralWidget(widget_central);
	QVBoxLayout *vboxLayout = new QVBoxLayout( widget_central );
	vboxLayout->setSpacing(0);
	vboxLayout->setMargin(0);
	vboxLayout->addLayout(hboxLayout);
	vboxLayout->addWidget(browserWidget);
	
	QHBoxLayout *hboxLayout2 = new QHBoxLayout;
	hboxLayout2->addWidget(gameWidget);
	hboxLayout2->addWidget((QWidget*)gameWidget->parameterWidget);
	vboxLayout->addLayout(hboxLayout2);
	
	setWidgetInitialStrings();		//no translation yet
	
	//create the status bar by displaying an empty message
	setPermanentStatusMessage("");
	
	//start the browser
	currentMode=BrowserDisplayMode;
	historyIndex=-1;
	
	//create the sound engine
	//and add a copy of the pointer in game widget and browserwidget
	//note : this is bad practice, but easier than adding signals and slots everywhere
	soundEngine = new SoundEngine(this);
	gameWidget->soundEngine = soundEngine;
	browserWidget->soundEngine = soundEngine;
	
	//InitialSize
	resize(Global::InitialWidth,Global::InitialHeight);
	
	//read the settings file (userName)
	readSettings();
	
	//try to download the translation file
	//finishStartProcess() will be called after the download (whether it succeeds or fails)
	downloadInitFiles();
}

//Displays a status message.
//Currently, we use the status bar, but an object that appears/disappears would be better
void FlukzWidget::setPermanentStatusMessage(const QString& message) {
	//statusBar() is a function of QMainWindow that returns a pointer to the statusBar
	//it also creates the statusBar at the first call
	statusBar()->showMessage(message);
}

//Displays a temporary status message, that is cleared after 0.6 seconds
void FlukzWidget::setTemporaryStatusMessage(const QString& message) {
	statusBar()->showMessage(message, 1000);	//unit is ms
}

//we set all the widget strings in this function
//this will allow us to dynamically change the initial translation by calling this function again
void FlukzWidget::setWidgetInitialStrings() {
	userLoginAction->setText(tr("Login..."));
	userPageAction->setText(tr("Personal page"));
	userButton->setToolTip(tr("User login"));
	editButton->setToolTip(tr("Start edition"));
	saveButton->setToolTip(tr("Save the changes"));
	viewMetaAction->setText(tr("View Meta information"));
	saveCopyAction->setText(tr("Save a copy in a local file..."));
	renamePageAction->setText(tr("Rename current page..."));
	deletePageAction->setText(tr("Delete current page..."));
	uploadFileAction->setText(tr("Upload a file..."));
	toolButton->setToolTip(tr("Tool functions"));
	refreshButton->setToolTip(tr("Refresh Page"));
}

//download the files needed to initialize the application
//(1) the translation into the local language of the strings displayed in the application interface
//(2) the correspondence between language short names and the name of language (en -> English)
//this function is called at the end of the interface creation
void FlukzWidget::downloadInitFiles() {
	//set askedUrl to flukz server url
	//needed to download the translation file, but also to initialize correctly the login widget
	QString urlName = QString("fkz://%1/").arg(Connection::serverDefaultHostName);
	QUrl url(urlName);
	FileRepository::setCurrentHostName(url.host());		//used to avoid file collisions of different servers
	askedUrl = url;
	QStringList initFiles;
	
	QString locale = QLocale::system().name();
	if(locale.left(2)!="en") {
		QString localeTranslationFile = FileRepository::langFolderName + "flukz" + FlukzWidget::flukzVersionNumber + "_" + locale.left(2) + ".qm";
		initFiles += localeTranslationFile;
	}
	
	QString languageNamesFile = FileRepository::langFolderName + "languageNames";
	initFiles += languageNamesFile;
	
	currentOperation = DownloadInitFiles;
	clientObject->download(initFiles, url.host());
}

//try to load the translation if the system langage is different from english
void FlukzWidget::loadTranslation() {
	QString locale = QLocale::system().name();
	if(locale.left(2)!="en") {
		QString langFileName = FileRepository::langFolderName + "flukz" + FlukzWidget::flukzVersionNumber + "_" + locale.left(2) + ".qm";
		translator = new QTranslator;
		bool success = translator->load( FileRepository::getAbsoluteFileName(langFileName) );
		if(success) QCoreApplication::installTranslator(translator);
	}
}

//end of the start process, called after we have tried to download the initialization files
void FlukzWidget::finishStartProcess() {
	loadTranslation();
	setWidgetInitialStrings();
	browserWidget->readLanguageFile(FileRepository::langFolderName + "languageNames");
	
	//ask for the user login if the user has left flukz in a connected state
	//when he leaves the application in a connected state, the userName is saved in settings file
	//and it has been read before arriving here
	if(!userName.isEmpty()) userLogin();
	
	QUrl startUrl(startPage());
	browserWidget->openLink(startUrl);
}

//address of the page opened when Flukz is launched
QString FlukzWidget::startPage() {
	QString homePage = tr("en/Homepage");
	#ifdef DEBUG
		//in debug mode, use localhost by default
		//(to avoid doing tests directly on main server by error)
		return QString("fkz://localhost/") + homePage;
	#else
		return QString("fkz://%1/").arg(Connection::serverDefaultHostName) + homePage;
	#endif
}

//called when the user press Enter
void FlukzWidget::adressBarChanged() {
	QUrl url(addressBar->text());	//url in the address bar
	pageAsked(url);
}

//called when the user changes the text in the address bar
void FlukzWidget::AddressBarTextEdited(){
	if(!isEditing) {
		//when we are not editing a page, the refresh button becomes a forward arrow
		refreshButton->setIcon(QIcon(":/images/forwardarrow.png"));
	}
}

//called when a link is clicked in the browser, or when the user press enter
void FlukzWidget::pageAsked(const QUrl& url) {
	if(url.hasQueryItem("revert")) {
		askedUrl = url;
		askedFileName = url.path();
		revertTargetRev = url.queryItemValue("revert").toInt();
		revertCurrentPage();
	} else
	if (PageChangeAuthorization()) {
		//delete the history in the forward direction
		forwardArrow->setEnabled(false);
		historyIndex++;
		while(historyIndex < urlHistory.size() ) {
			urlHistory.removeAt(historyIndex);
		}
		urlHistory.append(url);
		if(historyIndex>0) backwardArrow->setEnabled(true);
		setUrl(url);
	}
}

//called when a different page is asked by the game widget
//to quit a game, go to the next level, or download the high scores
void FlukzWidget::gamePageAsked(const QString& page) {
	if(page.isEmpty()) {
		//case of simply quitting the game widget
		//go back to the first page that was not a game level
		if(historyIndex>0) {
			for(int i = historyIndex-1; i>=0; i--){
				if(!urlHistory.at(i).toString().contains(".fkz")){
					//if the url does not contain ".fkz", we consider that it is not a level page
					//note/warning : don't give directly urlhistory as an argument
					//  it is modified in pageAsked, and so a crash can happen
					QUrl newUrl(urlHistory.at(i).toString());
					pageAsked(newUrl);
					return;
				}
			}
		}
		
		//if we have not found a previous page which is not a level, go back to the start page
		QUrl startUrl(startPage());
		pageAsked(startUrl);
	} else
	if(page.endsWith(FileRepository::highScoreResource)) {
		//game widget asked the updated version of the high scores
		//this is not a request to move to the page of the high scores
		currentOperation = RefreshGameScore; 
		clientObject->download(page, askedUrl.host());
	} else {
		//game widget asked to go the next level
		QUrl nextLevelUrl(page);
		QUrl currentUrl(addressBar->text());
		QUrl finalUrl = BrowserWidget::deduceAbsoluteUrl(currentUrl, nextLevelUrl);
		finalUrl.setQueryItems(currentUrl.queryItems());	//set the same queries (?action=play)
		pageAsked(finalUrl);
	}
}

bool FlukzWidget::PageChangeAuthorization() {
	// Preliminary operations before refreshing the page.
	switch(currentMode) {
	case GamePlayMode:
		gameWidget->stopGame();
	break;
	case GamePreparePlayMode:
	case GamePrepareEditMode:
	case GamePrepareCreditsMode:
		//we have not finished to prepare the game play/edit mode
		//so we don't refresh.
		return false;
	break;
	case BrowserDisplayMode:
		//Nothing to do.
	break;
	case BrowserEditMode:
	case GameEditMode:
	case GameTestMode:
		if(isEditing) {
			//we need to stop the edit mode before going to a new page
			if(!toggleEditMode()) return false;	//the user canceled the action
		}
	break;
	}
	//Put the "refresh" icon on the refresh button
	//this is necessary if the refresh icon was changed in a forward arrow
	refreshButton->setIcon(QIcon(":/images/refresh.png"));
	refreshButton->setToolTip("Refresh");
	return true;
}

void FlukzWidget::setUrl(const QUrl& url) {
	//stop any remaining sound (for example if a sound was currently read in browserwidget)
	soundEngine->stopAllSounds();
	soundEngine->enableSound(false);
	
	addressBar->setText(url.toString());
	
	//set the host name in filerepository (used to avoid file collisions of different servers)
	FileRepository::setCurrentHostName(url.host());
	
	currentOperation = DownloadFile;
	askedUrl = url;
	askedFileName = url.path();
	if(askedFileName.startsWith('/')) askedFileName.remove(0, 1);
	
	//TODO : meaning of this code ?
	if(askedFileName.endsWith(".html") ||
	   askedFileName.endsWith(FileRepository::logResource)  ||
	   askedFileName.endsWith(FileRepository::talkResource) ||
	   askedFileName.endsWith(FileRepository::metaResource) ||
	   FileRepository::isOldFile(askedFileName)
	) {
		viewMetaAction->setVisible(false);
	} else {
		viewMetaAction->setVisible(true);
	}
	
	clientObject->download(url.path(), url.host());
}

//this function is called when clientObject has finished the download of the requested files
//We define in which order the application downloads what
//currentOperation describes the state of the application in respect of the download process
//currentMode describes a more general state of the application
//failedFiles : list of the files that were not donwloaded successfully
void FlukzWidget::downloadEnd(const QStringList& failedFiles) {
	//If a game play or a game edition begins, it will resize the window
	//We save the window size in order to bring it back to the user when he has finished playing
	if (currentMode==BrowserDisplayMode || currentMode==BrowserEditMode) {
		userWindowSize=size();
		userWindowGeometry=saveState();
	}
	if(currentOperation == WaitState) {
		//nothing to do
	} else
	if(currentOperation == DownloadFile || currentOperation == DownloadTemplateFile) {
		if(!failedFiles.isEmpty()) {
			//the file was not found
			if(currentOperation!=DownloadTemplateFile &&
			   askedFileName.endsWith(".fkz") && askedUrl.hasQueryItem("createFrom")
			) {
				//we are in the process of creating a new level from a given template level
				//(and we are not already downloading the template)
				currentOperation = DownloadTemplateFile;
				QStringList dependencies;
				dependencies += askedUrl.queryItemValue("createFrom");
				clientObject->download(dependencies, askedUrl.host());
			} else {
				//default case
				connectionError(Message::FileNotFoundError);
			}
			return;
		}
		
		//the file was downloaded correctly
		if(askedFileName.endsWith(".fkz")) {
			QString levelFileName;
			if(askedUrl.hasQueryItem("createFrom")) {
				levelFileName = askedUrl.queryItemValue("createFrom");
			} else {
				levelFileName = askedFileName;
			}
			gameWidget->setLevel(levelFileName);
			
			if(askedUrl.queryItemValue("action") == "edit" || askedUrl.hasQueryItem("createFrom")) {
				currentMode = GamePrepareEditMode;
			} else
			if(askedUrl.queryItemValue("action") == "credits") {
				currentMode = GamePrepareCreditsMode;
			} else {
				currentMode = GamePreparePlayMode;
			}
			
			currentOperation = DownloadGameDependency;
			QStringList dependencies;
			if(currentMode == GamePrepareCreditsMode) {
				//dependencies to display the credits (true = isCredits)
				dependencies = gameWidget->filesDependencyList(true);
				browserWidget->setGameCreditsList(dependencies);	//prepare the browser
			} else {
				//usual dependencies to play or edit a level (false = is not credits)
				dependencies = gameWidget->filesDependencyList(false);
				
				//display a "loading..." message
				browserWidget->setVisible(false);
				gameWidget->setVisible(true);
				resize(sizeHint());
				gameWidget->setFocus();
				gameWidget->displayLoadingMessage();
			}
			clientObject->download(dependencies, askedUrl.host());
		} else {
			//this is a file for the browser
			QStringList dependencies;
			bool isEditable = FileRepository::isEditablePage(askedFileName);
			editButton->setEnabled(isEditable);
			dependencies = browserWidget->filesDependencyList(askedUrl);
			
			if(dependencies.isEmpty()) {
				currentOperation = WaitState;
				setBrowserMode();
				browserWidget->downloadEnd(askedUrl);
			} else {
				currentOperation = DownloadWikiDependency;
				clientObject->download(dependencies, askedUrl.host());
			}
		}
	} else
	if(currentOperation == DownloadGameDependency) {
		currentOperation = WaitState;
		
		if(currentMode == GamePrepareCreditsMode) {
			//all the dependencies for the credits have been downloaded
			//we display the credits in the browser
			//note : the list of files for credits was already given to the browser with setGameCreditsList()
			setBrowserMode();
			browserWidget->downloadEnd(askedUrl);
			return;
		}
		
		if(currentMode == GamePrepareEditMode) {
			currentMode = GameEditMode;
		} else
		if(currentMode == GamePreparePlayMode) {
			currentMode = GamePlayMode;
			editButton->setEnabled(false); // Forbid the edition of the level during the game.
		}
		
		if(currentMode == GamePlayMode) {
			gameWidget->startGame();
		} else
		if(currentMode == GameEditMode) {
			toggleEditMode();
			gameWidget->startEdit();
			resize(sizeHint().width(), size().height());	//resize only the width to avoid scrollbars
		}
	} else
	if(currentOperation == RefreshGameScore) {
		currentOperation = WaitState;
		gameWidget->refreshHighScore();
	} else
	if(currentOperation == DownloadInitFiles) {
		//the download of the translation file is finished
		//(successful or not, there is no problem)
		currentOperation = WaitState;
		finishStartProcess();
	} else
	if(currentOperation == DownloadWikiDependency) {
		//the download of the dependencies of a html file is finished
		//(successfull or not, there is no problem)
		currentOperation = WaitState;
		setBrowserMode();
		browserWidget->downloadEnd(askedUrl);
	}
}

//this function is called after a connection error in clientObject
void FlukzWidget::connectionError(Message::ErrorCode errorCode) {
	if(currentOperation == DownloadInitFiles) {
		//an error occured during the initial download of the translation file
		//we simply finish the start process
		currentOperation = WaitState;
		finishStartProcess();
		return;
	}
	
	bool isEditable = false;
	if(errorCode!=Message::FileNotFoundError) {
		//block the edit button for any other error than "file not found"
		editButton->setEnabled(false);
	} else {
		//if the file is not found, the page can be editable in some cases
		isEditable = FileRepository::isEditablePage(askedFileName);
		editButton->setEnabled(isEditable);
	}
	//display the error in the browser
	setBrowserMode();
	currentOperation = WaitState;
	browserWidget->connectionError(errorCode, askedUrl, isEditable);
}

//function called when the refresh/preview button is clicked
void FlukzWidget::refreshButtonClicked() {
	switch(currentMode) {
	case BrowserDisplayMode:
	case GamePlayMode:
	case GamePreparePlayMode:
	case GamePrepareEditMode:
	case GamePrepareCreditsMode:
		adressBarChanged();
		break;
	case GameEditMode:
		currentMode = GameTestMode;
		refreshButton->setIcon(QIcon(":/images/stop.png"));
		gameWidget->trigGameTest();
		break;
	case GameTestMode:
		currentMode = GameEditMode;
		refreshButton->setIcon(QIcon(":/images/run.png"));
		gameWidget->trigGameTest();
		break;
	case BrowserEditMode:
		//in the browser edit mode, a click on the refresh button launches the preview
		previewAsked();
	}
}

//function called when the toggle edit button is clicked
//when closing the edition of a game, we need to go back to the previous page
void FlukzWidget::editButtonClicked() {
	if(currentMode==GameEditMode || currentMode == GameTestMode) {
		if(toggleEditMode()) goToPreviousPage();
	} else
	if(currentMode==BrowserDisplayMode && askedFileName.endsWith(".fkz")) {
		//we are currently displaying a fkz file in the browser
		//it means that the file was not existing, we can create it from a given template file
		createLevelFile();
	} else {
		toggleEditMode();
	}
}

//return true if the toggle was done
//updateScreen is used to block the update of the browser widget after saving the changes
//   in this particular case, we need to download first the new content before switching back to the html mode
bool FlukzWidget::toggleEditMode(bool updateScreen /*default=true*/) {
	if(isEditing) {
		if(!isSaved) {
			//some changes have not been saved, confirmation dialog
			int cancel=QMessageBox::question(this, tr("Confirmation"),
			    tr("Your changes have not been saved.\nDo you really want to quit the edit mode ?"),
				tr("Quit edit mode"),
				tr("Cancel"), QString(), 1, 1);
			if(cancel) return false;
			saveButton->setEnabled(false);
			isSaved=true;
		}
		isEditing=false;
		editButton->setIcon(QIcon(":/images/edit.png"));
		editButton->setToolTip(tr("Start edition"));
		refreshButton->setIcon(QIcon(":/images/refresh.png"));
		refreshButton->setToolTip("Refresh");
		switch(currentMode) {
			case GameEditMode:
			case GameTestMode:
				setBrowserMode();	//it will close the game edit mode
				break;
			case BrowserEditMode:
			default:
				if(updateScreen) browserWidget->stopEditing();
				currentMode = BrowserDisplayMode;
		}
	} else {
		if(currentMode==BrowserDisplayMode) {
			browserWidget->startEditing();
			currentMode = BrowserEditMode;
		}
		isEditing=true;
		editButton->setEnabled(true);
		editButton->setIcon(QIcon(":/images/exit.png"));
		editButton->setToolTip(tr("Stop edition"));
		if(currentMode==GameEditMode) {
			refreshButton->setIcon(QIcon(":/images/run.png"));
			refreshButton->setToolTip("Test the level");
		} else {
			refreshButton->setIcon(QIcon(":/images/preview.png"));
			refreshButton->setToolTip("Preview");
		}
	}
	
	return true;
}

void FlukzWidget::closeGameTestMode() {
	if(currentMode == GameTestMode) {
		currentMode = GameEditMode;
		refreshButton->setIcon(QIcon(":/images/run.png"));
	}
}

//called in editing mode when the user makes a modification
//  game editing mode : when the scene changes (add sprite, move sprite, etc.)
//  browser editing mode : when the text changes
void FlukzWidget::fileChanged() {
	//the user changed something in the file, activate the save button
	if(isEditing && isSaved) {
		saveButton->setEnabled(true);
		isSaved=false;
	}
}

//save on the server the local changes of the page/level
void FlukzWidget::saveCurrentFile() {
	if(isSaved) {
		//error : the save button should not be available if the file is already saved
		saveButton->setEnabled(false);
		return;
	}
	
	//save the page or the level in a temporary file
	if(currentMode==GameEditMode || currentMode==GameTestMode) {
		gameWidget->saveTemporaryFile();
	} else
	if(currentMode==BrowserEditMode) {
		browserWidget->saveTemporaryFile();
	}
	
	//open the commit dialog
	CommitWidget commitWidget(this);
	commitWidget.setDialogType(CommitWidget::CommitDialog, userLoggedIn);
	commitWidget.setTargetFile(askedFileName);
	connect(&commitWidget, SIGNAL(askCommit(Message)), this, SLOT(startCommit(Message)) );
	connect(clientObject, SIGNAL(networkMessage(Message)), &commitWidget, SLOT(networkMessage(Message)) );
	
	int accept=commitWidget.exec();
	if(!accept) return;
	
	//refresh the interface (accept is called only in case of success)
	saveButton->setEnabled(false);
	isSaved = true;
	
	if(currentMode==BrowserEditMode) {
		//we toggle the edit mode with a special parameter that blocks the update of the browser
		//because we want to download first the new file before reading it again in the browser
		toggleEditMode(/*updateScreen*/ false);
		setUrl(askedUrl);
	}
}

//save on the server a score reached by the player
void FlukzWidget::commitGameScore(const QString& startLevelName, int score) {
	SimpleDialogWidget simpleDialogWidget(this);
	simpleDialogWidget.setParameters(tr("Save a high score") , tr("Congratulations. This is a new high score.\nName for the high score:"), tr("Save"));
	if(!userName.isEmpty()) {
		simpleDialogWidget.setDefaultAnswer(userName);
	} else {
		simpleDialogWidget.setDefaultAnswer("");
	}
	int accept=simpleDialogWidget.exec();
	if(!accept) {
		//the player did not want to save his high score
		//we simply download the current high score
		QString neededHighScoreFile = FileRepository::getHighScoreFileName(startLevelName);
		gamePageAsked(neededHighScoreFile);
		return;
	}
	
	//get the name that the user entered
	QString scoreName = simpleDialogWidget.getUserAnswer();
	Message scoreMessage;
	scoreMessage.fileName = FileRepository::getHighScoreFileName(startLevelName);
	scoreMessage.commandCode = Message::MergeData;
	scoreMessage.data = QString("%1").arg(score);
	scoreMessage.msg = scoreName;
	
	currentOperation = RefreshGameScore;	//remember the current state of the application
	
	startCommit(scoreMessage);
}

void FlukzWidget::setBrowserMode() {
	if(currentMode != BrowserDisplayMode) {
		if(currentMode == GamePlayMode) {
			gameWidget->stopGame();
		} else
		if(currentMode == GameEditMode) {
			gameWidget->closeEditMode();
		} else
		if(currentMode == GameTestMode) {
			gameWidget->trigGameTest();		//close the test first
			gameWidget->closeEditMode();
		}
		currentMode = BrowserDisplayMode;
		gameWidget->stopAllTimers();
		gameWidget->setVisible(false);
		browserWidget->setVisible(true);
		resize(userWindowSize);
	}
}

void FlukzWidget::goToPreviousPage() {
	if(historyIndex>0) {
		if (PageChangeAuthorization()){
			historyIndex--;
			if(historyIndex==0) backwardArrow->setEnabled(false);
			forwardArrow->setEnabled(true);
			setUrl(urlHistory.at(historyIndex));
		}
	}
}

void FlukzWidget::goToNextPage() {
	if(historyIndex+1 < urlHistory.size()) {
		if (PageChangeAuthorization()){
			historyIndex++;
			if(historyIndex==urlHistory.size()-1) forwardArrow->setEnabled(false);
			backwardArrow->setEnabled(true);
			setUrl(urlHistory.at(historyIndex));
		}
	}
}

void FlukzWidget::previewAsked() {
	browserWidget->startPreview();
	currentMode = BrowserEditMode;
}

//-------------------------- Tools -----------------------

//View the meta information of the current file
//(this function is available for files that are not html pages)
void FlukzWidget::viewMeta() {
	if(isEditing) {
		//we need to stop the edit mode before displaying the history
		if(!toggleEditMode()) return;	//the user canceled the action
	}
	
	QUrl metaUrl = askedUrl;
	QList<QPair<QString, QString> > noQueryItems;
	metaUrl.setQueryItems(noQueryItems);
	metaUrl.setPath( askedUrl.path() + FileRepository::metaResource);
	pageAsked(metaUrl);
}

//Upload a file on the flukz server
void FlukzWidget::uploadFile() {
	if(isEditing) {
		//we need to stop the edit mode before uploading a file
		if(!toggleEditMode()) return;	//the user canceled the action
	}
	
	CommitWidget commitWidget(this);
	commitWidget.setDialogType(CommitWidget::UploadDialog, userLoggedIn);
	connect(&commitWidget, SIGNAL(askCommit(Message)), this, SLOT(startCommit(Message)) );
	connect(clientObject, SIGNAL(networkMessage(Message)), &commitWidget, SLOT(networkMessage(Message)) );
	
	int accept=commitWidget.exec();
	if(!accept) return;
	
	QUrl metaUrl = askedUrl;
	QList<QPair<QString, QString> > noQueryItems;
	metaUrl.setQueryItems(noQueryItems);
	metaUrl.setPath( commitWidget.serverFileName->text() + FileRepository::metaResource);
	pageAsked(metaUrl);
}

void FlukzWidget::createLevelFile() {
	SimpleDialogWidget simpleDialogWidget(this);
	simpleDialogWidget.setParameters(tr("Create a new level from a template") , tr("Name of the template level:"), tr("Create"));
	simpleDialogWidget.setDefaultAnswer("template.fkz");
	int accept=simpleDialogWidget.exec();
	if(!accept) return;
	
	//get the template level name entered by the user in the simple dialog widget
	QString templatePageName = simpleDialogWidget.getUserAnswer();
	
	QUrl newUrl = askedUrl;
	QList<QPair<QString, QString> > queryItems;
	QPair<QString, QString> query;
	query.first="createFrom";
	query.second=templatePageName;
	queryItems.append(query);
	newUrl.setQueryItems(queryItems);
	pageAsked(newUrl);
}

//Save the current page in a file
//usefull when we want to make a copy of a file somewhere without looking for it in the cache
void FlukzWidget::saveCopy() {
	//ask to the user the name of the file where a copy will be saved
	QString localCopyFileName = QFileDialog::getSaveFileName(this, tr("Save a copy of the current page as..."), QDir::homePath());
	
	//copy the file of the cache where the current page is saved
	QString urlFileName = askedUrl.path();
	QString localCacheName = FileRepository::getAbsoluteFileName(urlFileName);
	
	//TODO : save a copy of the target file selected by the user if it already exists
	QFile::remove(localCopyFileName);
	QFile::copy(localCacheName, localCopyFileName);
}

//Delete the current page from the server
//this function is not available for normal users 
void FlukzWidget::deleteCurrentPage() {
	if(isEditing) {
		//we need to stop the edit mode
		if(!toggleEditMode()) return;	//the user canceled the action
	}
	
	CommitWidget commitWidget(this);
	commitWidget.setDialogType(CommitWidget::DeleteDialog, userLoggedIn);
	connect(&commitWidget, SIGNAL(askCommit(Message)), this, SLOT(startCommit(Message)) );
	connect(clientObject, SIGNAL(networkMessage(Message)), &commitWidget, SLOT(networkMessage(Message)) );
	
	int accept=commitWidget.exec();
	if(!accept) return;
	
	QUrl userUrl = askedUrl;
	QString currentPath = askedUrl.path();
	if(currentPath.startsWith('/')) currentPath.remove(0, 1);
	QList<QPair<QString, QString> > noQueryItems;
	userUrl.setQueryItems(noQueryItems);
	userUrl.setPath(FileRepository::trashFolderName + currentPath);
	pageAsked(userUrl);
}

//Rename the current page to another name
void FlukzWidget::renameCurrentPage() {
	if(isEditing) {
		//we need to stop the edit mode
		if(!toggleEditMode()) return;	//the user canceled the action
	}
	
	CommitWidget commitWidget(this);
	commitWidget.setDialogType(CommitWidget::RenameDialog, userLoggedIn);
	connect(&commitWidget, SIGNAL(askCommit(Message)), this, SLOT(startCommit(Message)) );
	connect(clientObject, SIGNAL(networkMessage(Message)), &commitWidget, SLOT(networkMessage(Message)) );
	
	int accept=commitWidget.exec();
	if(!accept) return;
	
	QUrl userUrl = askedUrl;
	QList<QPair<QString, QString> > noQueryItems;
	userUrl.setQueryItems(noQueryItems);
	QString newPageName = FileRepository::nameWithoutSpaces(commitWidget.serverFileName->text());
	userUrl.setPath(newPageName);
	pageAsked(userUrl);
}

//Revert the current page to an old revision
void FlukzWidget::revertCurrentPage() {
	if(isEditing) {
		//we need to stop the edit mode
		if(!toggleEditMode()) return;	//the user canceled the action
	}
	
	CommitWidget commitWidget(this);
	commitWidget.commitMessage.fileRevision=(qint16) revertTargetRev;
	commitWidget.setDialogType(CommitWidget::RevertDialog, userLoggedIn);
	connect(&commitWidget, SIGNAL(askCommit(Message)), this, SLOT(startCommit(Message)) );
	connect(clientObject, SIGNAL(networkMessage(Message)), &commitWidget, SLOT(networkMessage(Message)) );
	
	int accept=commitWidget.exec();
	if(!accept) return;
	
	//revert succeded, so we display the new current page
	QUrl userUrl = askedUrl;
	QList<QPair<QString, QString> > noQueryItems;
	userUrl.setQueryItems(noQueryItems);
	pageAsked(userUrl);
}

//start a commit (upload, rename, delete, revert, create user, authenticate user)
void FlukzWidget::startCommit(Message message) {
	if(message.commandCode==Message::RenameFile ||
	   message.commandCode==Message::DeleteFile ||
	   message.commandCode==Message::RevertFile) {
		message.fileName=askedFileName;
	}
	if(!message.fileName.isEmpty()) {
		message.fileName = clientObject->cleanString(message.fileName);
	}
	clientObject->addMessageAndConnect(message, askedUrl.host());
}

void FlukzWidget::userLogin() {
	if(!userLoggedIn){
		LoginWidget loginWidget(this);
		connect(&loginWidget, SIGNAL(askCommit(Message)), this, SLOT(startCommit(Message)) );
		connect(clientObject, SIGNAL(networkMessage(Message)), &loginWidget, SLOT(networkMessage(Message)) );
		
		if(!userName.isEmpty()) loginWidget.promptUserName(userName);
		int accept=loginWidget.exec();
		if(!accept) {
			userName.clear();	//userName possibly not empty if we used promptUserName
			return;
		}
		
		//set the user+password in the client widget used for network communications
		userName = loginWidget.userNameEdit->text();
		QString password = loginWidget.passwordEdit->text();
		clientObject->setUser( userName, password);
		
		//update the user button and interface title
		userButton->setIcon(QIcon(":/images/identified-user.png"));
		userLoginAction->setText(tr("Logout"));
		userPageAction->setEnabled(true);
		setWindowTitle(QString("Flukz ") + flukzVersionNumber + "  -  " + tr("Logged as user: %1").arg(userName) );
		userLoggedIn = true;
		
		//in case of a new user, display the personal page
		if(loginWidget.isAccountCreation) {
			userPersonalPage();
		}
	} else {
		userName.clear();
		clientObject->userLogout();
		userButton->setIcon(QIcon(":/images/anonymous-user.png"));
		userLoginAction->setText(tr("Login..."));
		userPageAction->setEnabled(false);
		setWindowTitle(QString("Flukz ") + flukzVersionNumber );
		userLoggedIn = false;
	}
}

//Display the user personal page
void FlukzWidget::userPersonalPage() {
	if(isEditing) {
		//we need to stop the edit mode before displaying the user personal page
		if(!toggleEditMode()) return;	//the user canceled the action
	}
	
	QUrl userUrl = askedUrl;
	QList<QPair<QString, QString> > noQueryItems;
	userUrl.setQueryItems(noQueryItems);
	userUrl.setPath( QDir::cleanPath(FileRepository::userFolderName + userName) );
	pageAsked(userUrl);
}

//return the absolute file name of the settings file
//it is simply stored at the root of FileRepository
QString FlukzWidget::settingsFileName() {
	QDir directory(FileRepository::repositoryPath);
	return directory.absoluteFilePath("flukzSettings.ini");
}

//read a setting file
void FlukzWidget::readSettings() {
	QSettings settings(settingsFileName(), QSettings::IniFormat);
	
	//read the user name if existing
	userName = settings.value("userName", /*default value*/ "").toString();
	
	//restore the main window state (size and position, maximized or not)
	userWindowGeometry = settings.value("windowGeometry").toByteArray();
	restoreGeometry(userWindowGeometry);
}

//write a setting file
void FlukzWidget::writeSettings() {
	QSettings settings(settingsFileName(), QSettings::IniFormat);
	
	settings.setValue("userName", userName);		//save the user name if connected
	
	//save the window save and position to restore them at the nect Flukz start
	//if we are in browser display mode, get the current size
	//otherwise, we use the last saved windows size
	if(currentMode == BrowserDisplayMode) {
		userWindowSize = size();
		userWindowGeometry = saveGeometry();
	}
	settings.setValue("windowGeometry", userWindowGeometry);
}

//reimplement QWidget::closeEvent (called when the user closes the application)
//we save the settings before closing the application
void FlukzWidget::closeEvent(QCloseEvent *event) {
	writeSettings();
	event->accept();
}
