/***************************************************************************
 *   Copyright (C) 2011-2014 by Flukz Team                                 *
 *   Contact : see our web site http://flukz.org/                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef FLUKZ_WIDGET_H
#define FLUKZ_WIDGET_H

#include "../global.h"
#include "../networkengine/message.h"

#include <QMainWindow>
#include <QUrl>

class QLineEdit;
class QAction;
class QPushButton;
class ClientObject;
class BrowserWidget;
class GameWidget;
class QTranslator;
class QCloseEvent;
class CommitData;
class SoundEngine;

class FlukzWidget : public QMainWindow {
	Q_OBJECT
  public:
	explicit FlukzWidget( QWidget *parent=0 );

  private:
	static QString flukzVersionNumber;
	
	enum FlukzMode {GamePlayMode=0, GamePreparePlayMode=1, GameEditMode=2, GamePrepareEditMode=3, 
				GamePrepareCreditsMode=4,
				GameTestMode=5,
				BrowserDisplayMode=6, BrowserEditMode=7};
	FlukzMode currentMode;
	
	QLineEdit *addressBar;
	QAction *viewMetaAction, *saveCopyAction, *renamePageAction, *deletePageAction, *uploadFileAction;
	QPushButton *backwardArrow, *forwardArrow, *toolButton, *editButton, *saveButton, *userButton, *refreshButton;
	bool isEditing, isSaved, userLoggedIn;
	
	BrowserWidget *browserWidget;
	GameWidget *gameWidget;
	ClientObject *clientObject;
	
	QTranslator *translator;
	
	SoundEngine *soundEngine;		//sound engine to play and mix sounds from wav files
	
	enum OperationValue { WaitState=0, DownloadInitFiles=1, DownloadFile=2, DownloadTemplateFile=3,
						   DownloadGameDependency=4, DownloadWikiDependency=5,
						   RefreshGameScore=6 };
	OperationValue currentOperation;
	
	QString askedFileName;
	QUrl askedUrl;
	int revertTargetRev;	//TODO : this variable here is a bad design
	
	QList<QUrl> urlHistory;
	int historyIndex;
	
	QString commitComment;
	
	QAction *userLoginAction, *userPageAction;
	QString userName;
	
	bool toggleEditMode(bool updateScreen=true);		//switch between display mode and edit mode
	bool PageChangeAuthorization();
	QString settingsFileName();							//return the absolute file name of the settings file
	
	QSize userWindowSize;			//size of the main window (used to restore the size after a game ends)
	QByteArray userWindowGeometry;		//size and position data about the state of the main window 
	
  private slots:
	void adressBarChanged();
	void pageAsked(const QUrl&);
	void gamePageAsked(const QString&);
	void setUrl(const QUrl& url);
	void downloadEnd(const QStringList&);
	void connectionError(Message::ErrorCode errorCode);
	
	void editButtonClicked();
	void refreshButtonClicked();
	void fileChanged();
	void saveCurrentFile();
	
	void setBrowserMode();
	
	void goToPreviousPage();
	void goToNextPage();
	void AddressBarTextEdited();
	
	void setPermanentStatusMessage(const QString& message);
	void setTemporaryStatusMessage(const QString& message);
	void previewAsked();
	
	void closeGameTestMode();
	void commitGameScore(const QString &startLevelName, int score);
	
	//tools
	void viewMeta();
	void createLevelFile();
	void saveCopy();
	void renameCurrentPage();
	void deleteCurrentPage();
	void revertCurrentPage();
	void uploadFile();
	void startCommit(Message message);
	
	//user login
	void userLogin();
	void userPersonalPage();
	
	//initialization step (in particular, translations)
	void loadTranslation();
	void setWidgetInitialStrings();
	void downloadInitFiles();
	void finishStartProcess();
	QString startPage();
	
	//read-write settings file
	void readSettings();
	void writeSettings();
	
  protected:
	void closeEvent(QCloseEvent *event);	//override QApplication virtual function closeEvent
};

#endif
