/***************************************************************************
 *   Copyright (C) 2011-2014 by Flukz Team                                 *
 *   Contact : see our web site http://flukz.org/                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "loginwidget.h"
#include "../networkengine/message.h"
#include "../networkengine/filerepository.h"
#include "../global.h"

#include <QHBoxLayout>
#include <QPushButton>
#include <QGridLayout>
#include <QLabel>
#include <QLineEdit>

LoginWidget::LoginWidget(QWidget* parent) : QDialog( parent) {
	setWindowTitle(tr("User identification"));
	
	//--------------horizontal layout with Login and cancel button------------
	QWidget *buttons = new QWidget( this );
	QHBoxLayout *buttonsLayout = new QHBoxLayout(buttons);
	buttonsLayout->setMargin(0);
	
	//Save button and Cancel button (with focus)
	buttonsLayout->addStretch(1);
	
	confirmButton = new QPushButton( tr("Login"), buttons );
	confirmButton->setAutoDefault(false);
	connect(confirmButton, SIGNAL(clicked()), this, SLOT(completeValidation()));
	buttonsLayout->addWidget(confirmButton);
	
	QPushButton *pushButton = new QPushButton( tr("Cancel"), buttons );
	pushButton->setAutoDefault(false);
	connect(pushButton, SIGNAL(clicked()), this, SLOT(reject()));
	buttonsLayout->addWidget(pushButton);
	
	buttonsLayout->addStretch(1);
	
	//------------------vertical layout --------------------------
	QGridLayout *gridLayout = new QGridLayout( this );
	gridLayout->setSpacing(5);
	gridLayout->setMargin(5);
	int row=0;
	
	titleLabel0 = new QLabel;
	titleLabel0->setText(tr("If you don't have an account:"));
	createButton = new QPushButton( tr("Create a new account"), buttons );
	createButton->setAutoDefault(false);
	connect(createButton, SIGNAL(clicked()), this, SLOT(createAccountDialog()));
	gridLayout->addWidget( titleLabel0, row, 0, 1, 2);
	gridLayout->addWidget( createButton, row, 2);
	row++;
	
	//label and lineEdit for the user name
	titleLabel1 = new QLabel;
	titleLabel1->setText(tr("User name:"));
	userNameEdit = new QLineEdit;
	connect(userNameEdit, SIGNAL(returnPressed()), this, SLOT(userNameValidation()));
	gridLayout->addWidget( titleLabel1, row, 0);
	gridLayout->addWidget( userNameEdit, row, 1);
	explainLabel0 = new QLabel;
	explainLabel0->setText("");
	gridLayout->addWidget( explainLabel0, row, 2);
	row++;
	
	//label and lineEdit for the password
	titleLabel2 = new QLabel;
	titleLabel2->setText(tr("Password:"));
	passwordEdit = new QLineEdit;
	passwordEdit->setEchoMode(QLineEdit::Password);
	connect(passwordEdit, SIGNAL(returnPressed()), this, SLOT(passwordValidation()));
	gridLayout->addWidget( titleLabel2, row, 0);
	gridLayout->addWidget( passwordEdit, row, 1);
	explainLabel1 = new QLabel;
	explainLabel1->setText("");
	gridLayout->addWidget( explainLabel1, row, 2);
	row++;
	
	//label and lineEdit for the password confirmation (when creating an account)
	titleLabel3 = new QLabel;
	titleLabel3->setText(tr("Confirm Password:"));
	titleLabel3->setVisible(false);
	passwordConfirmEdit = new QLineEdit;
	passwordConfirmEdit->setEchoMode(QLineEdit::Password);
	passwordConfirmEdit->setVisible(false);
	connect(passwordConfirmEdit, SIGNAL(returnPressed()), this, SLOT(passwordConfirmValidation()));
	isAccountCreation = false;
	gridLayout->addWidget( titleLabel3, row, 0);
	gridLayout->addWidget( passwordConfirmEdit, row, 1);
	explainLabel2 = new QLabel;
	explainLabel2->setText("");
	gridLayout->addWidget( explainLabel2, row, 2);
	row++;
	
	gridLayout->addWidget( buttons, row, 0, 1, 3 );
	
	//-------------------other initializations ------------------------------
	//no context hint button (by setting to 0 the bit of the hint button in the window flag)
	setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
	
	userNameEdit->setFocus();
	waitingNetworkAnswer = false;	//the widget is not locked
}

bool LoginWidget::userNameIsValid() {
	//username must be at least 4 characters long
	bool isValid = true;
	
	QString userName = userNameEdit->text();
	QString cleanUserName = FileRepository::formatUserName(userName);
	if(cleanUserName!=userName) {
		userNameEdit->setText(cleanUserName);
	}
	
	if(cleanUserName.size() < 1) {
		isValid = false;
		explainLabel0->setText(tr("At least 1 character needed."));
	}
	
	if(!isValid) {
		userNameEdit->setFocus();
	} else {
		explainLabel0->setText("");
	}
	return isValid;
}

bool LoginWidget::passwordIsValid() {
	if(passwordEdit->text().size()<6) {
		explainLabel1->setText(tr("At least 6 characters needed."));
		passwordEdit->setFocus();
		return false;
	}
	
	//password must not contain spaces (note : maybe obsolete since hashes have been introduced)
	if(passwordEdit->text().contains(' ')) {
		explainLabel1->setText(tr("Spaces are forbidden."));
		passwordEdit->setFocus();
		return false;
	}
	
	return true;
}

bool LoginWidget::passwordConfirmIsValid() {
	if(passwordConfirmEdit->text() != passwordEdit->text()) {
		explainLabel2->setText(tr("Confirmation doesn't match\nwith the password."));
		passwordConfirmEdit->setFocus();
		return false;
	}
	return true;
}

void LoginWidget::userNameValidation() {
	if(userNameIsValid()) passwordEdit->setFocus();
}

void LoginWidget::passwordValidation() {
	if(passwordIsValid()) {
		explainLabel1->setText("");
		if(isAccountCreation) {
			passwordConfirmEdit->setFocus();
		} else{
			completeValidation();
		}
	}
}

void LoginWidget::passwordConfirmValidation() {
	if(passwordConfirmIsValid()) {
		explainLabel2->setText("");
		completeValidation();
	}
}

void LoginWidget::completeValidation() {
	if(!userNameIsValid()) return;
	if(!passwordIsValid()) return;
	if(isAccountCreation) {
		if(!passwordConfirmIsValid()) return;
		startUserCreation();
	} else {
		startUserAuthentication();
	}
}

void LoginWidget::createAccountDialog() {
	isAccountCreation = true;
	titleLabel0->setVisible(false);
	createButton->setVisible(false);
	titleLabel3->setVisible(true);
	passwordConfirmEdit->setVisible(true);
	userNameEdit->setFocus();
	setWindowTitle(tr("Account creation"));
	confirmButton->setText(tr("Create"));
}

//used when a user reconnects after leaving flukz in a connected state
//we ask the password again
void LoginWidget::promptUserName(const QString& userName) {
	userNameEdit->setText(userName);
	passwordEdit->setFocus();
}

void LoginWidget::startUserCreation() {
	//if we are waiting a network answer, we ignore new validations
	//this can happen if the return key is pressed multiple times
	if(waitingNetworkAnswer) return;
	waitingNetworkAnswer = true;	//lock the widget until the network answer
	confirmButton->setEnabled(false);
	
	commitMessage.commandCode=Message::CreateUser;
	commitMessage.setUser(userNameEdit->text(), passwordEdit->text());
	emit askCommit(commitMessage);
}

void LoginWidget::startUserAuthentication() {
	//if we are waiting a network answer, we ignore new validations
	//this can happen if the return key is pressed multiple times
	if(waitingNetworkAnswer) return;
	waitingNetworkAnswer = true;	//lock the widget until the network answer
	confirmButton->setEnabled(false);
	
	commitMessage.commandCode=Message::AuthenticateUser;
	commitMessage.setUser(userNameEdit->text(), passwordEdit->text());
	emit askCommit(commitMessage);
}

//this is connected to ClientObject::networkMessage
//we receive here the messages received by clientObject
void LoginWidget::networkMessage(Message message) {
	if(message.statusCode==Message::ActionSuccess &&
	   (message.commandCode==Message::CreateUser || message.commandCode==Message::AuthenticateUser) ) {
		//user creation or authentication was successfull
		accept();
	} else
	if(message.commandCode==Message::CreateUser && message.statusCode==Message::UserAlreadyExists) {
		explainLabel0->setText(tr("The user %1 already exists.").arg(message.msg));
		waitingNetworkAnswer = false;	//unlock the widget
		confirmButton->setEnabled(true);
	} else
	if(message.commandCode==Message::CreateUser && message.statusCode==Message::ActionFailed) {
		explainLabel0->setText(tr("User creation failed for an unknown reason."));
		waitingNetworkAnswer = false;	//unlock the widget
		confirmButton->setEnabled(true);
	} else
	if(message.commandCode==Message::AuthenticateUser && message.statusCode==Message::ActionFailed) {
		explainLabel1->setText(tr("Wrong password."));
		waitingNetworkAnswer = false;	//unlock the widget
		confirmButton->setEnabled(true);
	}
}
