/***************************************************************************
 *   Copyright (C) 2011-2014 by Flukz Team                                 *
 *   Contact : see our web site http://flukz.org/                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "networkengine/serverwidget.h"
#include "networkengine/serverobject.h"
#include "browseengine/flukzwidget.h"
#include "soundengine/soundengine.h"

#include <QApplication>
#include <QTime>
#include <time.h>
#include <math.h>

void main_server(int argc, char **argv, bool isMaster);
void main_graphicServer(int argc, char **argv, bool isMaster);
void main_graphic(int argc, char **argv);
void main_help();
void main_error();

//entry point of Flukz
int main( int argc, char **argv ) {
	srand(time(NULL));	//initialisation so that rand() won't give always the same values
	std::string s;
	switch(argc) {
	  case 2 : s=(std::string) argv[1];
		if     (s=="-gs" || s=="--graphicserver") main_graphicServer(argc, argv, false);
		else if(s=="-s" || s=="--server") main_server(argc, argv, false);
		else if(s=="-mgs" || s=="--mastergraphicserver") main_graphicServer(argc, argv, true);
		else if(s=="-ms" || s=="--masterserver") main_server(argc, argv, true);
		else if(s=="-h" || s=="--help")   main_help();
		#ifdef MACX
		//special hack for mac platform (there is a program id parameter when run on double-click)
		else if (s.substr(0,4)=="-psn")   main_graphic(argc, argv);
		#endif
		else if(s=="-mute" || s=="--mute") {
			//posibility of running Flukz without the sound
			//it can be useful to test on Linux platforms if pulseaudio is not available
			SoundEngine::disableSoundEngine();
			main_graphic(argc, argv);
		}
		else main_error();
		break;
	  case 1 :
		main_graphic(argc, argv);
		break;
	  default :
		main_error();
	}
	
	return EXIT_SUCCESS;
}

//Qt graphical interface for the client
void main_graphic(int argc, char **argv) {
	QApplication a( argc, argv );
	qsrand(QTime(0,0,0).secsTo(QTime::currentTime()));
	
	//show the main widget of the application
	FlukzWidget *hm = new FlukzWidget();
	hm->show();
	
	a.exec();
}

//server with graphical interface
void main_graphicServer(int argc, char **argv, bool isMaster) {
	QApplication a(argc, argv);
	qsrand(QTime(0,0,0).secsTo(QTime::currentTime()));
	
	ServerWidget *hm = new ServerWidget(isMaster);
	hm->show();
	
	hm->exec();
}

//server from the command line
void main_server(int argc, char **argv, bool isMaster) {
	QCoreApplication a(argc, argv);
	qsrand(QTime(0,0,0).secsTo(QTime::currentTime()));
	
	ServerObject *hm = new ServerObject(isMaster);
	hm->activateCommandLineDisplay();
	hm->launchServer();
	
	a.exec();		//starts the Qt main event loop
}

//available options in command line
void main_help() {
	std::cout << "Available parameters for Flukz :" << std::endl
	     << "nothing                       :" << "  " << "start the client" << std::endl
		 << "-mute or --mute               :" << "  " << "start the client without any sound support" << std::endl
	     << "-s  or --server               :" << "  " << "launch a slave server in command line mode" << std::endl
	     << "-gs or --graphicserver        :" << "  " << "launch a slave server with graphic display" << std::endl
		 << "-ms  or --masterserver        :" << "  " << "launch a master server in command line mode" << std::endl
	     << "-mgs or --mastergraphicserver :" << "  " << "launch a master server with graphic display" << std::endl
	     << "-h  or --help                 :" << "  " << "print  this help list" << std::endl;
}

//unavailable parameter
void main_error(){
	std::cout << "This parameter is not valid." << std::endl;
	main_help();
}
