/***************************************************************************
 *   Copyright (C) 2011-2014 by Flukz Team                                 *
 *   Contact : see our web site http://flukz.org/                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef LEVEL_H
#define LEVEL_H

#include "spriteparameter.h"
#include "levelparameter.h"

#include<QMultiMap>
#include <QList>
#include <QString>
#include <QObject>

class QGraphicsItem;

//represent the 2d coordinates of a sprite
//used as the keys of the map storing the sprites
class SpriteCoordinate {
  public:
	qreal x;
	qreal y;
	
	//the sprites are sorted by y coordinate first
	//TODO : this cannot be used in horizontal levels 
	bool operator<(const SpriteCoordinate& a) const {
		if(y != a.y) {
			return y < a.y;
		} else {
			return x < a.x;
		}
	}
};

class Level {
  public:
	Level();
	void init();
	void readFile(const QString &filename);
	void writeFile(const QString &filename);
	QStringList filesDependencyList(bool isCreditsDependencies);
	
	//--------- level description -------------------------
	QMultiMap<SpriteCoordinate, SpriteParameter> spriteMap;				//map used in game mode (y_pos, spriteParam)
	SpriteKindParameter getSpriteKindParameter(int spriteIndex);
	void setSpriteKindParameter(int spriteIndex, SpriteKindParameter skp);
	SpriteParameter getSpriteDefaultParameters(int spriteIndex);
	QString getPixmapName(int spriteIndex);
	int getPixmapWidth(int spriteIndex);
	int getPixmapHeight(int spriteIndex);
	bool hasNextLevel();
	QString nextLevelName();
	
	QStringList getSoundFileList();								//list of absolute file names of the sounds
	LevelParameter levelParam;
	
	//--------- variables used during the game -------------
	qreal y_front;	//y value corresponding to the upper limit of the scene
	qreal level_execute_end;	//y value of the end of the level during execution
	QMultiMap<SpriteCoordinate, SpriteParameter>::iterator currentSprite; //currentSprite points to the last sprite that has not appeared on the screen
	
	//used in edit mode to save the changes
	void createLevelFromScene(QList<QGraphicsItem*> itemList);
	
	//used in test mode to create a partial level starting at y coordinate
	void createPartialLevelFromScene(QList<QGraphicsItem*> itemList, qreal y_start);
	
  private:
	QString currentFileName;
	
	void overWrite();								//write in the current file name
	void addSpriteToMap(SpriteParameter p);
};

#endif
