/***************************************************************************
 *   Copyright (C) 2011-2014 by Flukz Team                                 *
 *   Contact : see our web site http://flukz.org/                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef LEVELPARAMETER_H
#define LEVELPARAMETER_H

#include "spriteparameter.h"
#include <vector>

#include <QStringList>

//this class stores all the relevent parameters of a given level
class LevelParameter {
 public:
	LevelParameter();
	qreal y_end;				//last possible value of y_front (i.e length of the level)
	qreal scrollingSpeed;
	unsigned int itemCount;		//total number of item (also used to give a new id to a new item)
	std::string backgroundColor;
	
	//link to the next level
	std::string nextLevelFileName;
	
	//list of sound files used in the level. index 0 = music, index 1 = explosion
	std::vector<std::string> soundFileNames;
	
	//list of parameters for the different kind of sprites
	//the first sprite is always the player sprite
	std::vector<SpriteKindParameter> spriteKinds;
	
	QStringList filesDependencyList(bool isCreditsDependencies);		//list of the files needed for the level
	QString animationFileName(int spriteIndex, int animationIndex);
	int addSpriteKind(int askedId);					//add a new kind of sprite (a copy of the last one), and return its index
	
	//serialisation
	std::string toString() const;
	void fromString(const std::string& stringA, int version);
	std::string toUserString() const;
	void fromUserString(const std::string& stringA);
	bool limitValuesRange();
	static QList<Parameter> getParameters();		//list of parameters
	static std::string getSeparator();
};

#endif
