/***************************************************************************
 *   Copyright (C) 2011-2014 by Flukz Team                                 *
 *   Contact : see our web site http://flukz.org/                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

 
#ifndef SPRITE_H
#define SPRITE_H

#include "../global.h"
#include "spriteparameter.h"

#include <QGraphicsPixmapItem>

class Level;

//Sprite is a class to represent a sprite in the game
//note : it is important that Sprite inherits QObject before QGraphicsPixampItem
class Sprite : public QObject, public QGraphicsPixmapItem {
	Q_OBJECT
  public:
	Sprite();
	bool treatKey(int pressedKey, bool isPressed);
	void setParameters(SpriteParameter p);
	SpriteParameter getParameters();
	int getSpriteShape();
	int getLifePoints();
	void setSelectable(bool isSelectable);
	void setMovable(bool isMovable);
	void setMovableOutScene(bool isMovable);
	
	static void initKeyboardState();
	
	//ugly static pointer
	static Level* currentLevel;	//reference to the current level, used to know the pixmap associated to the sprites
	
	//static variable to know the number of steps to execute during an update of the scene
	static int stepsToExecute;
	
  signals:
	void fire(SpriteParameter paramFire);	//create fire sprites
	void destroy();							//ask to delete the item from the scene
	void changeScore(int variation);		//ask a change of score (usually when the sprite was destroyed)
	void lifePointsChanged(int lifePoints);	//signal that life points changed (used by the player sprite)
	void askSound(int soundFile);			//require a sound file given by its index
	void spriteSelected(SpriteParameter p);	//emit the parameters of the sprite when selected with the mouse
	void spriteMoved(SpriteParameter p);	//emit the parameters of the sprite when moved with the mouse
	
  protected:
	void advance(int step);
	void mousePressEvent(QGraphicsSceneMouseEvent *mouseEvent);
	void mouseMoveEvent(QGraphicsSceneMouseEvent *mouseEvent);
	//void mouseReleaseEvent(QGraphicsSceneMouseEvent *mouseEvent);
	
  private:
	SpriteParameter param;
	int spriteWidth;
	int spriteHeight;
	int fire_cycle;			//counter to know when to shoot the next fire
	bool isDestroyed;		//used to avoid sending multiple destroy signals for the same sprite
	int explodingCycle;		//counter to know when to switch to the next animation picture of the explosion
	int scoreValue;			//value of the sprite for the score (currently set to lifepoints)
	
	bool movableOutScene; 		//default = true; indicates if the sprite can move out of the scene 
	
	//used only for the player sprite controlled by keyboard
	static bool leftKeyPressed;
	static bool rightKeyPressed;
	static bool upKeyPressed;
	static bool downKeyPressed;
	
	//parameters of the player sprite that can be controlled by the keyboard
	static int playerFireSpeed;	//speed of player fire
	static int playerXSpeed;	//speed of player along x (when a key is pressed)
	static int playerYSpeed;	//speed of player along y (when a key is pressed)
	
	//used during a drog-drap with the mouse
	static QPointF mouseInitialCoordinates;
	static qreal spriteInitialX, spriteInitialY;
	
	//internal functions
	void setSpritePixmap(const QString& pixmapName);
	void explosionStart();
	void applyLimitPolicy(qreal& x_new, qreal& y_new);
};

#endif
