/***************************************************************************
 *   Copyright (C) 2011-2014 by Flukz Team                                 *
 *   Contact : see our web site http://flukz.org/                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef SPRITEPARAMETER_H
#define SPRITEPARAMETER_H

#include "../global.h"
#include "../tools/stringconverter.h"
#include "../tools/parameter.h"
#include <QStringList>

//this class stores all the relevent parameters of a sprite displayed in the scene
class SpriteParameter {
 public:
	SpriteParameter();
	int spriteShape;		//0=spaceShip, 1=fire, 2=enemy
	qreal x_pos;			//x coordinate
	qreal y_pos;			//y coordinate
	qreal z_pos;			//"depth" in the scene (similar to a z coordinate)
	qreal x_speed;			//speed along the x axis
	qreal y_speed;			//speed along the y axis
	int limitPolicy;		//behavior of the sprite when reaching the limit of the screen
	int collisionPolicy;	//behavior of the sprite when colliding with other items
	int fireIndex;			//index of the fire sprite
	int fireSpeed;			//0 = no current fire, fire is more slow with bigger values
	int lifePoints;			//number of times the sprite can be touched by fire without being destroyed
	
	//kind of sprite that created this sprite
	//used to remember if fire is from a friend or from an enemy
	int emittingSpriteShape;
	
	enum limitPolicies {BlockInScreen=0, DestroyWhenOut=1};
	enum collisionPolicies {NoCollision=0, Destroyed=1, DestroyedIndirectly=2};
	
	unsigned int id;	//unique id used to identify items
	
	void enforceEditSceneLimits(int spriteWidth, int spriteHeight);		//correct the x_pos and y_pos to ensure that the sprite is entirely in the scene
	
	//serialisation
	std::string toString() const;									//export a string representation (format = latestFormatVersion)
	void fromString(const std::string& stringA);					//init from a string representation (format = latestFormatVersion)
	void fromString(const std::string& stringA, int version);		//init from a string in a given format version (compatibility with older versions of Flukz)
	bool limitValuesRange();
	static QList<Parameter> getParameters(bool isRealSprite=true);		//list of parameters
	static std::string getSeparator();								//return the string command to initialize stringConverter correctly
	static int latestFormatVersion;
};

//support directly << and >> operators
StringConverter& operator<<(StringConverter& strConv, const SpriteParameter& p);
StringConverter& operator>>(StringConverter& strConv, SpriteParameter& p);

//general parameters describing a kind of sprites
class SpriteKindParameter {
  public:
	SpriteKindParameter();
	int spriteKind;							//integer used to distinguish the different possible sprites
	SpriteParameter defaultParameters;		//default parameters of a sprite of this kind
	std::string pixmapFileName;				//name of the pixmap file for this sprite
	int spriteWidth;						//width of the sprite (deduced from the pixmap)
	int spriteHeight;						//height of the sprite (deduced from the pixmap)
	std::string animationFileName;			//name of the pixmap file for the optional animation
	int animationLength;					//number of files in the animation
	int animationSpeed;						//delay between two images of the animation
	
	void detectSpriteDimensions();
	
	//serialisation
	std::string toString() const;								//export a string representation (format = latestFormatVersion)
	void fromString(const std::string& stringA);				//init from a string representation (format = latestFormatVersion)
	void fromString(const std::string& stringA, int version);	//init from a string in a given format version (compatibility with older versions of Flukz)
	std::string toLinearString() const;
	void fromLinearString(const std::string& stringA);			//init from a linear string representation
	bool limitValuesRange();
	static QList<Parameter> getParameters();					//list of information about the parameters
	static std::string getSeparator();							//return the string command to initialize stringConverter correctly
};

//support directly << and >> operators (we can use vector<SpriteKindParamer> or list<SpriteKindParameter> etc)
StringConverter& operator<<(StringConverter& strConv, const SpriteKindParameter& p);
StringConverter& operator>>(StringConverter& strConv, SpriteKindParameter& p);

#endif
