/***************************************************************************
 *   Copyright (C) 2011-2014 by Flukz Team                                 *
 *   Contact : see our web site http://flukz.org/                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef CLIENT_OBJECT_H
#define CLIENT_OBJECT_H

#include "connection.h"

#include <QObject>
#include <QStringList>

class ClientObject : public QObject {
	Q_OBJECT
	
  public:
	explicit ClientObject(QObject *parent = 0);
	
	QString cleanString(const QString& s);		//usefull function to clean a QString
	QStringList cleanString(const QStringList& listA);
	
  public slots:
	//download files, downloadEnd is emitted when the operation is finished 
	void download(const QString& fileName, const QString& serverName);
	void download(const QStringList& fileNames, const QString& serverName);
	
	//add a message to the list of messages and connect to the server
	void addMessageAndConnect(const Message& message, const QString& serverName);
	
	//set the user and password for all future communications
	void setUser(const QString & userName_init, const QString & password_init);
	
	//logout (unset the user and password)
	void userLogout();
	
  signals:
	void downloadEnd(const QStringList&);
	void connectionError(Message::ErrorCode);
	void networkMessage(Message message);
	void permanentStatusMessage(const QString& message);
	void temporaryStatusMessage(const QString& message);
	
  private slots:
	void connectToServer(const QString& serverName);
	void processMessage(const Message& message, Connection* connection);
	void askFileToServer(const QString& fileName, Connection* connection);
	void askFileHashToServer(const QStringList &fileNames, Connection* connection);
	void downloadNextFileWith(Connection* connection);
	void processError(QAbstractSocket::SocketError socketError);
	
  private:
	QStringList filesToDownload;
	QStringList filesNeededDownload;
	
	//messages that need to be send to the server
	QList<Message> waitingMessages;
	
	QStringList failedFiles;	//list of files that could not been downloaded correctly
	
	bool userIsAuthenticated;
	QString userName;
	QString password;
};

#endif
