/***************************************************************************
 *   Copyright (C) 2011-2014 by Flukz Team                                 *
 *   Contact : see our web site http://flukz.org/                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef CONNECTION_H
#define CONNECTION_H

#include <QObject>
#include <QTcpSocket>

#include "message.h"

class Connection : public QObject {
	Q_OBJECT
  public:
	explicit Connection(QObject *parent=0, QTcpSocket *socket=0);
	
	//default server port and hostName
	static quint16 serverDefaultPort;
	static QString serverDefaultHostName;
	
	//limit on the size of a single message
	static quint32 messageSizeLimit;
	
	//statistics about the download/upload bandwidth usage
	static unsigned long receivedGigaBytes;
	static unsigned long receivedBytes;
	static unsigned long sentGigaBytes;
	static unsigned long sentBytes;
	static void resetBandwidthStat();
	static void addReceivedBytes(quint32 receivedSize);
	static void addSentBytes(quint32 sentSize);
	
  public slots:
	void connectToHost(const QString& hostName, quint16 port);
	void disconnect();
	void sendData(const Message& message);
	
  signals:
	//signals that a message arrived, and in which connection object
	void messageReceived(const Message& message, Connection*);
	void error(QAbstractSocket::SocketError socketError);
	
	//signal emitted when we are connected or disconnected
	void connected();
	void disconnected();
	
  private:
	QTcpSocket *tcpSocket;
	bool isConnected;
	quint32 blockSize;
	quint16 networkProtocolVersion;
	bool magicNumberReceived;
	bool isWaitingData;
	
	static quint32 magicNumber;
	
  private slots:
	void initWaitState();
	void receiveData();
	void processError(QAbstractSocket::SocketError socketError);
	void socketConnected();
	void socketDisconnected();
};

#endif
