/***************************************************************************
 *   Copyright (C) 2011-2014 by Flukz Team                                 *
 *   Contact : see our web site http://flukz.org/                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef HIGHSCORE_FILE_H
#define HIGHSCORE_FILE_H

#include <QString>
#include <QMap>
#include <QPair>

class ScoreInfo {
  public:
	QString playerName;
	unsigned int timeT;
};

//high scores are sorted by couples (score, time)
typedef QPair<int, unsigned int> ScoreKey;

class HighScoreFile {
  public:
	HighScoreFile();
	HighScoreFile(const QString& localFileName);	//initialize the highscores from a given file
	
	bool addRecord(QString playerName, int score);	//return true if the score was really added
	QString getHtmlScoreList();
	
	//indicate if a new score can enter in the top
	bool isNewHighScore(int score);
	
	//read-write file operations
	void loadExistingFile(const QString& localFileName);
	void writeFile(const QString& localFileName);
	
  private:
	//the map is sorted by increasing scores and from recent times to older times
	//a highscore is considered better if it is a higher score, and for similar scores, it is older
	QMap<ScoreKey, ScoreInfo> scoreMap;
	int bottomScore;
	
	static unsigned int revertTime(unsigned int timeT);
	static const int maxScoresNumber = 20;
};

#endif
