/***************************************************************************
 *   Copyright (C) 2011-2014 by Flukz Team                                 *
 *   Contact : see our web site http://flukz.org/                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef MESSAGE_H
#define MESSAGE_H

#include <QStringList>

//the class Message is the basic object used to communicate between a client and a server
//but also inside the Flukz application itself

class Message {
  public:
	Message();
	void setUser(const QString& userInit, const QString& passwordInit);
	void setPreviousHash(const QString& localFileName);
	
	enum commands {BeginConnection=0, AskFileHash=1, FileHash=2,
				   AskFile=3, File=4, FileNotFound=5, FileReceived=6,
				   ReadingForbidden=7, WritingForbidden=8, WritingFailure=9,
				   DeleteFile=10, RenameFile=11, RevertFile=12,
				   CreateUser=13, AuthenticateUser=14,
				   MergeData=15, InvalidProtocolVersion=16
				};
	
	enum status {AskAction=0, ActionSuccess=1, ActionFailed=2, UserAlreadyExists=3};
	
	//connection error codes (used for communication inside the application itself, not on the network)
	enum ErrorCode {RemoteHostClosedError=0, HostNotFoundError=1, ConnectionRefusedError=2,
				 SocketTimeoutError=3, FileNotFoundError=4, InvalidProtocolError=5, UnknownError=6 };
	
	quint16 commandCode;	//command code
	quint16 statusCode;
	QString msg;			//message (for example the comment associated to most operations)
	QString fileName;		//name of a file
	QString fileNameB;		//name of a second file (for example in a rename)
	qint16 fileRevision;	//revision of a file (for example in a revert)
	QString previousHash;	//hash of the file before change (to check that change is done on the latest version)
	QString virtualPage;	//used to transmit the content of a virtual page (currently used for the status page)
	QString data;
	
	QStringList fileNameList;	//list of file names
	QStringList fileHashList;	//list of hash values
	
	bool userIsAuthenticated;
	QString user;
	QString password;
	
	//internal use, not transmitted on the network
	QString absoluteFileName;
	quint16 fileRepositoryStatus;
	
	//corresponding display message in the interface for each command code
	QString defaultDisplayString() const;
	
	//main functions to read/write on a QDataStream
	void writeOn(QDataStream& stream) const;
	void readFrom(QDataStream& stream);
	
	//flag to communicate with Connection (looks like an architecture problem here)
	bool userRejected;
	
  private:
	//functions to read/write a file on a QDataStream
	bool addFileContent(QDataStream& stream, const QString& asbsoluteFileName) const;
	bool addVirtualFileContent(QDataStream& stream, const QString& content) const;
	bool retrieveFileContent(QDataStream& networkDataStream, const QString& asbsoluteFileName);
	QString retrieveVirtualFileContent(QDataStream& stream);
};

//Stream operators to make serialization syntax easy on QDataStream
class QDataStream;
QDataStream& operator<<(QDataStream& stream, const Message& message);
QDataStream& operator>>(QDataStream& stream, Message& message);

#endif
