/**
 * Copyright (c) 2011 Nokia Corporation.
 *
 * Part of the Qt GameEnabler.
 */

#ifndef GEAUDIOOUT_H
#define GEAUDIOOUT_H

//#include <QThread>
#include <QObject>

//------comment Flukz patch---------
//Needed for the mutex to start/stop audioOut 
//------begin Flukz patch-----------
//#include <QMutex>
//------end Flukz patch-------------

#include "GEInterfaces.h"

// Forward declarations
class QAudioOutput;
class QIODevice;
class QuickTimer;

namespace GE {

//Flukz patch : complete rewrite of the class without threads
//there was stability problems on some platforms
class AudioOut : public QObject {
	Q_OBJECT
	
  public:
	AudioOut(AudioSource *source, QObject *parent = 0);
	virtual ~AudioOut();
	void enableSound(bool enable);
	
  public slots:
	void tick();
	
  protected: // Data
	QuickTimer * timer;		//owned
	QAudioOutput *m_audioOutput; // Owned
	QIODevice *m_outTarget; // Not owned
	AudioSource *m_source; // Not owned
	AUDIO_SAMPLE_TYPE *m_sendBuffer; // Owned
	int m_sendBufferSize;
	qint64 m_samplesMixed;
	bool m_isAudioOutputStarted;	//indicate if the AudioOutput is started or not
};

/*
//Previous class using threads
class AudioOut : public QThread
{
    Q_OBJECT

public: // Data types

    enum ThreadStates {
        NotRunning = 0,
        DoRun = 1,
        DoExit = 2
    };

public:
    AudioOut(AudioSource *source, QObject *parent = 0);
    virtual ~AudioOut();

public:
    bool usingThead() const { return m_usingThread; }

//------comment Flukz patch---------
//This is usually a source of problems to use slots in a class that derives from QThread 
//------begin Flukz patch-----------
//public slots:
    void tick();

//private slots:
    void audioNotify();
//------end Flukz patch-------------
	
protected: // From QThread
     virtual void run(); // For the threaded mode only!

protected: // Data
    QAudioOutput *m_audioOutput; // Owned
    QIODevice *m_outTarget; // Not owned
    AudioSource *m_source; // Not owned
    AUDIO_SAMPLE_TYPE *m_sendBuffer; // Owned
    int m_sendBufferSize;
    qint64 m_samplesMixed;
    int m_threadState;
    bool m_usingThread;
	
	//------comment Flukz patch---------
	//We have added the possibility to attach and detach the audioOutput manually
	//This is necessary when we want "real silence" (without any slight noise on active speakers) 
	//------begin Flukz patch-----------
	protected:
	bool m_isAudioOutputStarted;	//indicate if the AudioOutput is started or not
	QMutex m_audioOutMutex;			//used only to protect the m_outTarget object accessed by tick() and by enableSound() in different threads
	
	public:
	void enableSound(bool enable);
	//------end Flukz patch-------------
};
*/
} // namespace GE

#endif // GEAUDIOOUT_H
