/***************************************************************************
 *   Copyright (C) 2011-2014 by Flukz Team                                 *
 *   Contact : see our web site http://flukz.org/                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//Note : this class is based on the 

#ifndef SOUND_ENGINE_H
#define SOUND_ENGINE_H

#include <QObject>
#include <QStringList>

#include "GEAudioMixer.h"

namespace GE {
	class AudioOut;
	class AudioBuffer;
}

class SoundEngine : public QObject{
	Q_OBJECT
  public:
	explicit SoundEngine(QObject *parent = 0);
	virtual ~SoundEngine();
	
	//this static function can be called to disable completely the SoundEngine
	//it must be called before the creation of any SoundEngine
	static void disableSoundEngine();
	
  public slots:
	void playSound(int index, int repeateTimes);
	void stopAllSounds();
	void loadWavFiles(QStringList filesList);
	void unloadFiles();
	void enableSound(bool enable);
	
  private:
	//GE sound engine objects
	GE::AudioMixer mixer;				//mixer called when we play a sound 
	GE::AudioOut* audioOut;				//audio output linked to the mixer
	GE::AudioBuffer* *soundBufferArray;	//buffer of sounds (array of pointers) 
	int soundBufferCount;				//number of buffers in the array soundBufferArray
	
	//static variable to disable completely the use of SoundEngine
	//it is intended to be used when sound is not available on some platform
	//typically on Linux platforms without pulseaudio
	static bool isMute;
};

#endif
