/***************************************************************************
 *   Copyright (C) 2011-2014 by Flukz Team                                 *
 *   Contact : see our web site http://flukz.org/                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "difftool.h"
#include "../global.h"

#include <QRegExp>

//replace the html < and > symbols in the string with &lt; and &gt;
void DiffTool::formatToNoHtml(QString & wikiString) {
	wikiString.replace("<", "&lt;");
	wikiString.replace(">", "&gt;");
}

//format the diff string given in parameter to display it properly
//this is used when displaying the diff between two files
void DiffTool::formatDiffHtml(QString & wikiString) {
	//Note : this code is similar to one part of formatToStandardHtml
	//replace two newlines by <br/><br/>, and one newline by <br/>,
	// \s* means zero or any number of white spaces
	wikiString.replace(QRegExp("\\s*\\n\\n\\s*"), "<br /><br />");
	wikiString.replace(QRegExp("\\s*\\n\\s*"), "<br />");
}

//create the html corresponding to the difference between two files
void DiffTool::diffHtml(QString& oldString, QString& newString) {
	//replace < and > html symbols with &lt; and &gt;
	formatToNoHtml(oldString);
	formatToNoHtml(newString);
	
	findDiff(oldString, newString);
	colorDiff(oldString);
	colorDiff(newString);
	
	//----------- Old Code -------------
	/*
	//find the new line "\n" characters
	//in order to compare the blocks between the new lines
	int oldFileLength = oldString.length();
	int newFileLength = newString.length();
	int i_old=0, i_new=0, j_old=0, j_new=0;
	int oldBlockStartIndex=0;	//start of the block in the old file
	int oldBlockEndIndex=-1;	//end of the block in the old file
	int newBlockStartIndex=0;	//start of the block in the new file
	int newBlockEndIndex=-1;	//end of the block in the new file
	while(i_old<oldFileLength && i_new<newFileLength) {
		//initialize the blockEndIndex variables when we find the "\n" character
		if(oldBlockEndIndex==-1 && oldString[i_old]=='\n') {
			oldBlockEndIndex=i_old;
		}
		if(newBlockEndIndex==-1 && newString[i_new]=='\n') {
			newBlockEndIndex=i_new;
		}
		
		//check if we have found the end of the old and the new block
		if(oldBlockEndIndex!=-1 && newBlockEndIndex!=-1) {
			//if we have found the end of the blocks, color in red the differences
			j_old=oldBlockStartIndex;
			j_new=newBlockStartIndex;
			while(j_old<oldBlockEndIndex && j_new<newBlockEndIndex) {
				if(oldString[j_old]!=newString[j_new]) {
					newString.insert(j_new+1, "</FONT>");
					newString.insert(j_new, "<FONT color=#FF0000>");
					j_new += 28;
					newBlockEndIndex += 27;		//the block end index has changed because of the insertion
					newFileLength += 27;		//the length of the string has changed also
					j_old++;
				} else {
					j_new++;
					j_old++;
				}
			}
			
			if(j_new<newBlockEndIndex) {
				//the new line is longer than the old one.
				//color in red all the end of the new line
				newString.insert(newBlockEndIndex, "</FONT>");
				newString.insert(j_new, "<FONT color=#FF0000>");
				j_new += 28;				//note : useless, just for clean code
				newBlockEndIndex += 27;		//the block end index has changed because of the insertion
				newFileLength += 27;		//the length of the string has changed also
			}
			
			//when the differences have been computed, prepare the search of the next block
			oldBlockStartIndex = oldBlockEndIndex + 1;		//start the search after the end of the current block
			newBlockStartIndex = newBlockEndIndex + 1;
			oldBlockEndIndex = -1;			//reinitialize the end of the block (-1 means not found)
			newBlockEndIndex = -1;
			i_old = oldBlockStartIndex;		//initialize the loop variables to start the search at the beginning of the block
			i_new = newBlockStartIndex;
		} else {
			//we have not found the end of the blocks yet
			//simply increment the current index of the block (if needed)
			if(oldBlockEndIndex==-1) i_old++;
			if(newBlockEndIndex==-1) i_new++;
		}
	}
	
	if(i_new<newFileLength) {
		//the new file is longer, color all the end in red
		newString.insert(newFileLength-1, "</FONT>");
		newString.insert(i_new, "<FONT color=#FF0000>");
	}
	*/
	
	//Format both strings to a format that allows a simple visualization of the differences
	//In particular, we add line break with </br>
	formatDiffHtml(oldString);
	formatDiffHtml(newString);
}

//find the longest common substring of two given strings
//Note : this algorithm can be a limiting factor on large strings
void DiffTool::longestCommonSubstring(QString &string1, QString &string2, int& index1, int & index2, int& commonlength) {
	int le1=string1.length();
	int le2=string2.length();
	
	int M=std::min(le1,le2);
	
	//particular case of an empty string
	commonlength=0;
	index1=0;
	index2=0;
	if(M==0) return;
	
	//for performance reason, we limit the search to a 500 characters match
	int limitSearch=500;
	if(M>limitSearch) M=limitSearch;
	
	//note : this loop is valid only for M>0
	int pos=-1;
	QString strtofind;
	for(int i=M; i>=0; i--) {
		for(int j=0;j<le1-i;j++) {
			strtofind=string1.mid(j, i); //substring starting at j, of length i
			pos=string2.indexOf(strtofind);
			if(pos!=-1) {
				index1=j;
				index2=pos;
				commonlength=i;
				
				if(commonlength==limitSearch) {
					//we have found a common substring, but its length was limited by limitSearch
					//so we try to extend the common substring on the right
					bool end=false;
					while(!end && index1+commonlength<le1 && index2+commonlength<le2){
						if(string1[index1+commonlength]==string2[index2+commonlength]) {
							commonlength++;
						} else {
							end =true;
						}
					}
				}
				
				return;
			}
		}
	}
}

bool DiffTool::split(QString& string1, QString& string2) {
	int index1, index2, commonlength;
	longestCommonSubstring(string1, string2, index1, index2, commonlength);
	if (commonlength>2) {
		string1.insert(index1+commonlength, ">");
		string1.insert(index1, "<");
		string2.insert(index2+commonlength, ">");
		string2.insert(index2, "<");
		return true;
	} else {
		return false;
	}
}

//find the differences between the two strings
//common parts of size at least 3 are put inside <...> brackets
//an initial > and a terminal < are also added
//note : as a result, the different parts are found between >...< brackets
void DiffTool::findDiff(QString& string1, QString& string2) {
	//add initial > and ending < for algorithm inializations
	string1.insert(0,">");
	string1.append("<");
	string2.insert(0, ">");
	string2.append("<");
	
	//------------------------------------------------------
	//most real text files have a long common block at the beginning and at the end of the file
	//we use a very simple and quick function to find these first and last common blocks
	int length1 = string1.length();
	int length2 = string2.length();
	int minLength=std::min(length1-1, length2-1);	//don't include the last < character in the comparisons
	int start1=0, start2=0, end1=0, end2=0;  //start and end variables for the main algorithm
	int p=0;
	
	//Note : case of perfectly identical strings
	//find the common start block
	while(p<minLength && string1[p]==string2[p]) p++;
	if(p>1) {
		//put the common start block into <...> brackets
		string1.insert(p, ">");
		string1.insert(1, "<");
		length1+=2;		//size has increased of 2 characters
		string2.insert(p, ">");
		string2.insert(1, "<");
		length2 += 2;	//size has increased of 2 characters
		start1 = p;
		start2 = p;
	}
	
	//find the common end block
	minLength=std::min(length1-p-2, length2-p-2);		//to avoid possible overlap with the start block
	p=1;
	while(p<minLength && string1[length1-p]==string2[length2-p]) p++;
	if(p>2) {
		//put the common end block into <...> brackets
		string1.insert(length1-1, ">");
		string1.insert(length1-p+1, "<");
		string2.insert(length2-1, ">");
		string2.insert(length2-p+1, "<");
	}
	
	//----------------------------------------------------------
	//main algorithm
	bool success=true;
	QString currentBlock1, currentBlock2;
	while(start1!=-1) {
		end1=string1.indexOf('<',start1);	//next occurence of < after index start1
		end2=string2.indexOf('<', start2);	//next occurence of < after index start2
		currentBlock1=string1.mid(start1+1,end1-start1-1);
		currentBlock2=string2.mid( start2+1,end2- start2-1);
		success=split(currentBlock1, currentBlock2);
		string1=string1.left(start1+1)+currentBlock1+string1.mid(end1);
		string2=string2.left(start2+1)+ currentBlock2+string2.mid(end2);
		if(!success) {
			start1=string1.indexOf('<',start1);
			start2=string2.indexOf('<', start2);
		}
		start1=string1.indexOf('>',start1);
		start2=string2.indexOf('>', start2);
	}
}

//color in red the parts of the string that were found to be part of the diff
//we simply replace >...< brackets by <font>...</font> tags
void DiffTool::colorDiff(QString& string1) {
	int p=0;
	for(p=0; p<string1.length(); p++) {
		if(string1[p]=='<') {
			string1.insert(p+1, "/FONT>");
			p += 6;
		} else
		if(string1[p]=='>') {
			string1.insert(p, "<FONT color=#FF0000");
			p += 19;
		}
	}
}
