/***************************************************************************
 *   Copyright (C) 2011-2014 by Flukz Team                                 *
 *   Contact : see our web site http://flukz.org/                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "parameterwidget.h"
#include "stringconverter.h"

#include <QVBoxLayout>
#include <QTableWidget>
#include <QTableWidgetItem>
#include <QCloseEvent>

ParameterWidget::ParameterWidget( QWidget* parent) : QWidget( parent) {
	QVBoxLayout *vboxLayout = new QVBoxLayout( this );
	tableWidget = new QTableWidget(this);
	vboxLayout->addWidget(tableWidget);
	tableWidget->setMinimumWidth(400);
	tableWidget->setMinimumHeight(300);
	signalItemChange=false;
	connect(tableWidget, SIGNAL(itemChanged(QTableWidgetItem*)), this, SLOT(itemHasChanged(QTableWidgetItem*)));
	
	//initialization (no position has been saved yet)
	savedPosExist = false;
	
	// --- old code when ParameterWidget was a QDialog ---
	// --- could be useful again -------------------------
	//setWindowTitle("Parameters");
	//no context hint button (by setting to 0 the bit of the hint button in the window flag)
	//setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
	// ---- end of old code ------------------------------
}

void ParameterWidget::setParameterList(QString title, QList<Parameter> parametersInit, std::string separatorIni) {
	setWindowTitle(title);
	signalItemChange = false;	//don't react to item changes
	
	stringSeparator = separatorIni;
	parameters = parametersInit;
	tableWidget->clear();								//remove any existing items
	tableWidget->setRowCount(parameters.size());		//one row for each parameter
	tableWidget->setColumnCount(3);						//columns : parameter name - parameter type - parameter value
	
	QTableWidgetItem *item, *item2;
	item = new QTableWidgetItem(tr("Parameter"));
	tableWidget->setHorizontalHeaderItem(0, item);
	tableWidget->setColumnWidth(0,160);
	item = new QTableWidgetItem(tr("Type"));
	tableWidget->setHorizontalHeaderItem(1, item);
	tableWidget->setColumnWidth(1,70);
	item = new QTableWidgetItem(tr("Value"));
	tableWidget->setHorizontalHeaderItem(2, item);
	
	int i;
	for(i=0; i<parameters.size(); i++) {
		item = new QTableWidgetItem(parameters.at(i).name);
		item->setFlags(Qt::ItemIsSelectable|Qt::ItemIsEnabled);			//read-only
		tableWidget->setItem(i, 0, item);
		
		item = new QTableWidgetItem("");
		item->setFlags(Qt::ItemIsSelectable|Qt::ItemIsEnabled);		//read-only
		tableWidget->setItem(i, 1, item);
		
		item2 = new QTableWidgetItem("");
		item2->setToolTip(parameters.at(i).toolTip);
		tableWidget->setItem(i, 2, item2);
		
		Parameter::ParameterType type=parameters.at(i).parameterType;
		if(type==Parameter::IntType) {
			item->setText(tr("integer"));
			item2->setText("0");	//default value
		} else
		if(type==Parameter::BoolType) {
			item->setText(tr("boolean"));
			item2->setText("true");	//default value
		} else
		if(type==Parameter::QrealType) {
			item->setText(tr("real"));
			item2->setText("0");	//default value
		} else
		if(type==Parameter::StringType) {
			item->setText(tr("string"));
			item2->setText("");	//default value
		}
		
		if(!parameters.at(i).isEditable) {
			//this parameter is read-only
			item2->setFlags(Qt::ItemIsSelectable|Qt::ItemIsEnabled);
			item2->setBackgroundColor(Qt::lightGray);
		}
	}
}

//set the parameters of a given sprite (called after a click on a sprite in the edit mode)
//the use of StringConverter allows us to avoid any dependency to spriteParameter or levelParameter
void ParameterWidget::setParametersValue(std::string s) {
	//first, show the widget and move it back to its position before it was hidden
	if(!isVisible()) {
		show();
		if(savedPosExist) {
			resize(savedSize);
			move(savedPos);
		}
	}
	
	signalItemChange = false;	//don't react to item changes until the parameters are set
	
	StringConverter conv(s);
	conv << CvCd("UT") << CvCd(stringSeparator);	//use the stringSeparator
	
	int i;
	QTableWidgetItem *item;
	Parameter::ParameterType type;
	std::string itemString;
	for(i=0; i<parameters.size(); i++) {
		item=tableWidget->item(i,2);
		type=parameters.at(i).parameterType;
		if(type==Parameter::IntType) {
			conv >> itemString;
			item->setData(Qt::DisplayRole, QString::fromStdString(itemString));
		} else
		if(type==Parameter::BoolType) {
			bool itemBool;
			conv >> itemBool;
			if(itemBool == true) {
				item->setData(Qt::DisplayRole, QString("true"));
			} else {
				item->setData(Qt::DisplayRole, QString("false"));
			}
		} else
		if(type==Parameter::QrealType) {
			conv >> itemString;
			item->setData(Qt::DisplayRole, QString::fromStdString(itemString));
		} else
		if(type==Parameter::StringType) {
			conv >> itemString;
			item->setData(Qt::DisplayRole, QString::fromStdString(itemString));
		}
	}
	
	signalItemChange = true;	//react to item changes
}

//the content of one item has changed. We send a signal with the resulting string
void ParameterWidget::itemHasChanged(QTableWidgetItem * /*senderItem*/) {
	if(!signalItemChange) return;	//nothing to do if we don't signal item changes
	
	StringConverter conv;
	conv << CvCd("UT") << CvCd(stringSeparator);	//use ; separator
	
	int i;
	QTableWidgetItem *item;
	Parameter::ParameterType type;
	std::string itemString;
	for(i=0; i<parameters.size(); i++) {
		item=tableWidget->item(i,2);
		type=parameters.at(i).parameterType;
		if(type==Parameter::IntType) {
			conv << item->data(Qt::DisplayRole).toInt();
		} else
		if(type==Parameter::BoolType) {
			bool itemBool;
			if(item->data(Qt::DisplayRole).toString() == QString("true")) {
				itemBool = true;
			} else {
				itemBool = false;
			}
			conv << itemBool;
		} else
		if(type==Parameter::QrealType) {
			conv << item->data(Qt::DisplayRole).toReal();
		} else
		if(type==Parameter::StringType) {
			QString qs = item->data(Qt::DisplayRole).toString();
			std::string s = removeForbiddenCharacter(qs).toStdString();
			conv << s;
		}
	}
	
	emit contentChanged(conv.getString());
}

//the only accepted characters for strings are : a-z, A-Z, 0-9, '.' (46), '_' (95), '#' (35)
//'.' and '_' are common characters in file names
//'#' is needed for defining a custom Qt color
//this is because the current stringconverter object is base on std::string and not able to support utf8
QString ParameterWidget::removeForbiddenCharacter(const QString &targetString) {
	//targetString.erase( remove( targetString.begin(), targetString.end(), ']' ), targetString.end() );
	QString result;
	int n = targetString.size();
	for(int i=0; i<n; i++) {
		QChar qc = targetString[i];
		char c = qc.toAscii();
		if( (c>=48 && c<=57) || (c>=65 && c<=90) || (c>=97 && c<=122) || c==46 || c==95 || c==35) {
			result += qc;
		}
	}
	return result;
}

//event called when the user tries to close the widget
void ParameterWidget::closeEvent(QCloseEvent *event) {
	//never close really the parameter widget
	//only hide it
	//we also save the position (modeless qdialog are displayed in their original position after show)
	savedPos = pos();
	savedSize = size();
	savedPosExist = true;
	this->hide();
	event->ignore();
}
