/***************************************************************************
 *   Copyright (C) 2011-2014 by Flukz Team                                 *
 *   Contact : see our web site http://flukz.org/                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "quicktimer.h"

#include <QEvent>
#include <QCoreApplication>

void InternalTimer::run() {
	stopThread = false;
	while(!stopThread) {
		msleep(sleepTime);
		QEvent *event = new QEvent(QEvent::User);
		QCoreApplication::postEvent ( parentQuickTimer, event);
	}
}

void InternalTimer::startTimer(int msec, QuickTimer *parentTimer) {
	parentQuickTimer = parentTimer;
	sleepTime = msec;
	start();
}

void InternalTimer::stop() {
	//TODO : would be better with a mutex
	stopThread = true;
}

//=================================================================
// QuickTimer wrapper, this object is used in the main thread

QuickTimer::QuickTimer(QObject *parent) : QObject(parent) {
	
}

void QuickTimer::start(int msec) {
	timer.startTimer(msec, this);
}

void QuickTimer::stop() {
	timer.stop();
}

bool QuickTimer::event(QEvent * event) {
	if (event->type() == QEvent::User) {
		//we have received the signal from the internal timer
		emit timeout();
		event->accept();
		return true;
	} else {
		//transfer other events to default implementation
		return QObject::event(event);
	}
}
