/***************************************************************************
 *   Copyright (C) 2011-2014 by Flukz Team                                 *
 *   Contact : see our web site http://flukz.org/                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

 
#ifndef QUICKTIMER_H
#define QUICKTIMER_H

//QuickTimer is a temporary replacement to QTimer class on some platforms
//On linux, QTimer consumes 100% of the cpu when it ticks every 1ms
//https://bugreports.qt-project.org/browse/QTBUG-7618

//QuickTimer is inspired by the solution of Csaba Fekete on the post below
//  https://bugreports.qt-project.org/browse/QTBUG-6541
//however, horrible direct calls to QCoreApplication were needed to allow the communication
//between the threads

//QuickTimer class should be removed as soon as the mainstream version of Qt is confirmed
// to include a QTimer without this cpu bug

#include <QThread>

class QuickTimer;
class QEvent;

class InternalTimer : public QThread {
  public:
	void startTimer(int msec, QuickTimer *parentTimer);
	void stop();
	
  private:
	int sleepTime;	//micro-seconds
	bool stopThread;
	QuickTimer *parentQuickTimer;
	
  protected:
	void run();
};

class QuickTimer : public QObject {
	Q_OBJECT
  public:
	explicit QuickTimer(QObject *parent);
	bool event(QEvent * event);
	
  public slots:
	void start(int msec);
	void stop();
	
  signals:
	void timeout();
	
  private :
	InternalTimer timer;
};

#endif
