/***************************************************************************
 *   Copyright (C) 2011-2014 by Flukz Team                                 *
 *   Contact : see our web site http://flukz.org/                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "simpledialogwidget.h"

#include <QHBoxLayout>
#include <QGridLayout>
#include <QPushButton>
#include <QLabel>
#include <QLineEdit>
#include <QFileInfo>
#include <QFileDialog>
#include <QDir>


//TODO : this class will become later a bit more general
//TODO : we need to add the possibility to choose the language
//it is used currently for choosing the name of the new page or new level
SimpleDialogWidget::SimpleDialogWidget(QWidget* parent) : QDialog( parent) {
	//--------------horizontal layout with "Ok" button------------
	QWidget *buttons = new QWidget( this );
	QHBoxLayout *buttonsLayout = new QHBoxLayout(buttons);
	buttonsLayout->setMargin(0);
	
	//Create and Cancel button (with focus)
	buttonsLayout->addStretch(1);
	
	confirmButton = new QPushButton( tr("Ok"), buttons );
	confirmButton->setAutoDefault(false);
	connect(confirmButton, SIGNAL(clicked()), this, SLOT(accept()));
	buttonsLayout->addWidget(confirmButton);
	
	QPushButton *pushButton = new QPushButton( tr("Cancel"), buttons );
	pushButton->setAutoDefault(false);
	connect(pushButton, SIGNAL(clicked()), this, SLOT(reject()));
	buttonsLayout->addWidget(pushButton);
	
	buttonsLayout->addStretch(1);
	
	//------------------vertical layout --------------------------
	QGridLayout *gridLayout = new QGridLayout( this );
	gridLayout->setSpacing(5);
	gridLayout->setMargin(5);
	int row=0;
	
	//label and button to choose the file to upload
	titleLabel1 = new QLabel;
	titleLabel1->setText(tr("Page to create:"));
	pageName = new QLineEdit;
	
	gridLayout->addWidget( titleLabel1, row, 0);
	gridLayout->addWidget( pageName, row, 1);
	row++;
	
	//add the horizontal confirm/cancel buttons
	gridLayout->addWidget( buttons, row, 0, 1, 3 );
	
	//-------------------------------------------------
	//connects pressing the "return/enter" key to validation 
	connect(pageName, SIGNAL(returnPressed()), this, SLOT(accept()));
	
	//no context hint button (by setting to 0 the bit of the hint button in the window flag)
	setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
	
	pageName->setFocus();
}

//set the parameters of the dialog
//title = title displayed at the top of the dialog
//message = message displayed before the line edit
void SimpleDialogWidget::setParameters(QString title, QString message, QString confirmButtonName) {
	setWindowTitle(title);
	titleLabel1->setText(message);
	confirmButton->setText(confirmButtonName);
}

void SimpleDialogWidget::setDefaultAnswer(QString value) {
	pageName->setText(value);
}

//return the string entered by the user in the dialog
QString SimpleDialogWidget::getUserAnswer() {
	return pageName->text();
}
